@@ -29,6 +29,7 @@
 #include <lib/gnome-db-shortcut.h>
 #include "marshal.h"
 #include "object-selector.h"
+#include "item-selector.h"
 
 static void conf_manager_class_init (ConfManagerClass * class);
 static void conf_manager_init (ConfManager * conf);
@@ -311,9 +312,10 @@ build_main_window (ConfManager *conf)
 	conf->welcome = gtk_table_new (2, 2, FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (conf->welcome), GNOME_PAD * 2);
 	wid = gtk_label_new (NULL);
-	gtk_label_set_markup (GTK_LABEL (wid),
-			      _("<span style=\"italic\" size=\"larger\">Configure and open the connection "
-				"to begin your work.</span>"));
+	str = g_strdup_printf ("<span style=\"italic\" size=\"larger\">%s</span>",
+			       _("Configure and open the connection to begin your work."));
+	gtk_label_set_markup (GTK_LABEL (wid), str);
+	g_free (str);
 	gtk_table_attach_defaults (GTK_TABLE (conf->welcome), wid, 0, 1, 0, 1);
 	wid = gtk_image_new_from_file (PIXMAPDIR "/mergeant_foot.png");
 	gtk_table_attach_defaults (GTK_TABLE (conf->welcome), wid, 1, 2, 1, 2);
@@ -599,12 +601,6 @@ add_menus (GnomeApp * app, ConfManager * conf)
 
         /* connection menu */
         GnomeUIInfo conn_menu[] = {
-                {
-			GNOME_APP_UI_ITEM, N_("Connection preferences"), NULL,
-			options_config_cb,
-			conf, NULL,
-			GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_PREFERENCES,
-			0, 0, NULL},
                 {
 			GNOME_APP_UI_ITEM, N_("Connect Sql server"), NULL,
 			sql_conn_open_cb,
@@ -723,8 +719,7 @@ add_menus (GnomeApp * app, ConfManager * conf)
 	conf_manager_register_show_on_disconnect (conf, (db_menu[6]).widget);
 
 	conf_manager_register_sensitive_on_disconnect (conf, (conn_menu[0]).widget);
-	conf_manager_register_sensitive_on_disconnect (conf, (conn_menu[1]).widget);
-	conf_manager_register_sensitive_on_connect (conf, (conn_menu[2]).widget);
+	conf_manager_register_sensitive_on_connect (conf, (conn_menu[1]).widget);
 	conf_manager_register_sensitive_on_disconnect (conf, (file_menu[1]).widget);
 	conf_manager_register_sensitive_on_disconnect (conf, (file_menu[2]).widget);
 	conf_manager_register_sensitive_on_connect (conf, (file_menu[5]).widget);
@@ -788,10 +783,17 @@ debug_remove_field_cb (GtkWidget * widget, ConfManager * conf)
 }
 
 #include "canvas-query-view.h"
-static void os_cb (GtkWidget * widget, GObject *obj, gpointer data)
+static void os_cb (GtkWidget * widget, GObject *obj, gchar * data)
 {
-	g_print ("%p selected\n", obj);
+	g_print ("%s: %p selected\n", data, obj);
 }
+static gchar *type_get_name (ServerDataType *dt) {
+	return g_strdup (dt->sqlname);
+}
+static gchar *table_get_name (DbTable *table) {
+	return g_strdup (table->name);
+}
+
 static void
 debug_test_cb (GtkWidget * widget, ConfManager * conf)
 {
@@ -801,42 +803,120 @@ debug_test_cb (GtkWidget * widget, ConfManager * conf)
 	DbField *field;
 	Query *q;
 
-	if (! conf->top_query)
+	if (!conf->top_query)
 		return;
 
-	/* table = database_find_table_from_name (conf->db, "products"); */
+	if (1) {
+		/* table = database_find_table_from_name (conf->db, "products"); */
 /* 	field = db_table_find_field_by_name (table, "price"); */
-	/* q = query_find_from_xml_name (conf, NULL, "QU6"); */
+		/* q = query_find_from_xml_name (conf, NULL, "QU6"); */
 
-	dlg = gtk_dialog_new_with_buttons ("ObjectSelector test", NULL, 0,
-					   GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
+		dlg = gtk_dialog_new_with_buttons ("ObjectSelector test", NULL, 0,
+						   GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
 
-	/* hbox for all the widgets */
-	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), hbox, TRUE, TRUE, 0);
+		/* hbox for all the widgets */
+		hbox = gtk_hbox_new (FALSE, 0);
+		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), hbox, TRUE, TRUE, 0);
 
 	/* Object selector */
-	wid = object_selector_new (conf, OBJECT_SELECTOR_TABLES | OBJECT_SELECTOR_SEQUENCES | 
-				   OBJECT_SELECTOR_VIEWS | OBJECT_SELECTOR_DBFIELDS | OBJECT_SELECTOR_PROCS |
-				   OBJECT_SELECTOR_AGGS | OBJECT_SELECTOR_QUERIES | OBJECT_SELECTOR_QVIEWS |
-				   OBJECT_SELECTOR_DATA_TYPES,
-				   /* OBJECT_SELECTOR_PARAM_TOP_QUERY, q, */
+		wid = object_selector_new (conf, OBJECT_SELECTOR_TABLES | OBJECT_SELECTOR_SEQUENCES | 
+					   OBJECT_SELECTOR_VIEWS | OBJECT_SELECTOR_DBFIELDS | OBJECT_SELECTOR_PROCS |
+					   OBJECT_SELECTOR_AGGS | OBJECT_SELECTOR_QUERIES | OBJECT_SELECTOR_QVIEWS |
+					   OBJECT_SELECTOR_DATA_TYPES,
+					   /* OBJECT_SELECTOR_PARAM_TOP_QUERY, q, */
 /* 				   OBJECT_SELECTOR_PARAM_SUB_QUERIES, FALSE, */
-				   -1);
-	g_signal_connect (G_OBJECT (wid), "selection_changed",
-			  G_CALLBACK (os_cb), NULL);
-	/*object_selector_set_selection (OBJECT_SELECTOR (wid), G_OBJECT (field));*/
-	gtk_box_pack_start (GTK_BOX (hbox), wid, TRUE, TRUE, 10);
-	gtk_widget_set_usize (wid, 300, 400);
+					   -1);
+		g_signal_connect (G_OBJECT (wid), "selection_changed",
+				  G_CALLBACK (os_cb), NULL);
+		/*object_selector_set_selection (OBJECT_SELECTOR (wid), G_OBJECT (field));*/
+		gtk_box_pack_start (GTK_BOX (hbox), wid, TRUE, TRUE, 10);
+		gtk_widget_set_usize (wid, 300, 400);
 
 	/* GnomeDbBrowser */
 	/* wid = gnome_db_browser_new (); */
 /* 	gnome_db_browser_set_connection (GNOME_DB_BROWSER (wid), conf->srv->cnc); */
 /* 	gtk_box_pack_start (GTK_BOX (hbox), wid, TRUE, TRUE, 10); */
 
-	gtk_widget_show_all (dlg);
-	/*gtk_dialog_run (GTK_DIALOG (dlg));*/
-	/*gtk_widget_destroy (dlg);*/
+		gtk_widget_show_all (dlg);
+		/*gtk_dialog_run (GTK_DIALOG (dlg));*/
+		/*gtk_widget_destroy (dlg);*/
+	}
+	else {
+		GtkWidget *vbox;
+		dlg = gtk_dialog_new_with_buttons ("ItemSelector test", NULL, 0,
+						   GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
+		vbox = gtk_vbox_new (FALSE, GNOME_PAD);
+		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), vbox, TRUE, TRUE, 0);
+
+		/* static OFFSET */
+		/* menu */
+		wid = item_selector_new (ITEM_SELECTOR_MENU);
+		item_selector_set_static_content_offset (ITEM_SELECTOR (wid), 
+							 conf->db->tables, 
+							 G_STRUCT_OFFSET (DbTable, name));
+		gtk_box_pack_start (GTK_BOX (vbox), wid, TRUE, TRUE, 0);
+		g_signal_connect (G_OBJECT (wid), "selection_changed",
+				  G_CALLBACK (os_cb), NULL);
+
+		/* combo */
+		wid = item_selector_new (ITEM_SELECTOR_COMBO);
+		item_selector_set_static_content_offset (ITEM_SELECTOR (wid), 
+							 conf->db->tables, 
+							 G_STRUCT_OFFSET (DbTable, name));
+		gtk_box_pack_start (GTK_BOX (vbox), wid, TRUE, TRUE, 0);
+		g_signal_connect (G_OBJECT (wid), "selection_changed",
+				  G_CALLBACK (os_cb), NULL);
+
+		/* static FUNC */
+		/* menu */
+		wid = item_selector_new (ITEM_SELECTOR_MENU);
+		item_selector_set_static_content_func (ITEM_SELECTOR (wid), 
+						       conf->srv->data_types,
+						       (ItemSelectorFunc) type_get_name);
+		item_selector_set_selection_txt (ITEM_SELECTOR (wid), "int4");
+						       
+		gtk_box_pack_start (GTK_BOX (vbox), wid, TRUE, TRUE, 0);
+		g_signal_connect (G_OBJECT (wid), "selection_changed",
+				  G_CALLBACK (os_cb), NULL);
+
+		/* combo */
+		wid = item_selector_new (ITEM_SELECTOR_COMBO);
+		item_selector_set_static_content_func (ITEM_SELECTOR (wid), 
+						       conf->srv->data_types,
+						       (ItemSelectorFunc) type_get_name);
+		item_selector_set_selection_txt (ITEM_SELECTOR (wid), "int4");
+		gtk_box_pack_start (GTK_BOX (vbox), wid, TRUE, TRUE, 0);
+		g_signal_connect (G_OBJECT (wid), "selection_changed",
+				  G_CALLBACK (os_cb), NULL);
+
+
+		/* dynamic */
+		/* menu */
+		wid = item_selector_new (ITEM_SELECTOR_MENU);
+		item_selector_set_dyn_content (ITEM_SELECTOR (wid), G_OBJECT (conf->db),
+					       "table_created", "table_dropped",
+					       conf->db->tables,
+					       (ItemSelectorFunc) table_get_name);
+		item_selector_set_selection_txt (ITEM_SELECTOR (wid), "myview");
+						       
+		gtk_box_pack_start (GTK_BOX (vbox), wid, TRUE, TRUE, 0);
+		g_signal_connect (G_OBJECT (wid), "selection_changed",
+				  G_CALLBACK (os_cb), "MENU");
+
+		/* combo */
+		wid = item_selector_new (ITEM_SELECTOR_COMBO);
+		item_selector_set_dyn_content (ITEM_SELECTOR (wid), G_OBJECT (conf->db),
+					       "table_created", "table_dropped",
+					       conf->db->tables,
+					       (ItemSelectorFunc) table_get_name);
+		item_selector_set_selection_txt (ITEM_SELECTOR (wid), "myview");
+		gtk_box_pack_start (GTK_BOX (vbox), wid, TRUE, TRUE, 0);
+		g_signal_connect (G_OBJECT (wid), "selection_changed",
+				  G_CALLBACK (os_cb), "COMBO");
+
+
+		gtk_widget_show_all (dlg);
+	}
 }
 
 static void
@@ -893,6 +973,8 @@ about_cb (GtkWidget * widget, ConfManager * conf)
 			"Vivien Malerba <malerba@gnome-db.org>",
                         "Fernando Martins <fmartins@hetnet.nl>",
                         "Rodrigo Moya <rodrigo@gnome-db.org>",
+			"Carlos Perello Marin <carlos@gnome-db.org>", 
+			"Gonzalo Paniagua Javier <gonzalo@gnome-db.org>",
                         NULL
                 };
 
@@ -907,13 +989,20 @@ about_cb (GtkWidget * widget, ConfManager * conf)
                         "Ali Pakkan <apakkan@hotmail.com> Turk translations\n" \
                         "Christian Rose <menthos@menthos.com> Swedish translations\n" \
                         "Martin Lacko <lacko@host.sk> Slovak translation\n" \
-                        "Valek Filippov <frob@df.ru> Russian translation\n";
+                        "Valek Filippov <frob@df.ru> Russian translation\n" \
+			"Michal Bukovjan <bukm@centrum.cz> Czech translation\n" \
+			"Ole Laursen <olau@hardworking.dk> Danish translation\n" \
+			"Kjartan Maraas <kmaraas@online.no> Norwegian translation\n" \
+			"Zbigniew Chyla <translators@gnome.pl> Polish translation\n" \
+			"Pablo del Campo <pablodc@bigfoot.com> Spanish translation\n" \
+			"Christophe Merlet (RedFox) <redfox@eikonex.org> French translation\n" \
+			"Trinh Minh Thanh <tmthanh@linuxmail.org> Vietnamese translation\n";
 
 		/* FIXME: use a real GError here */
                 icon = gdk_pixbuf_new_from_file (PIXMAPDIR "/mergeant.png", NULL);
 		
 		dialog = gnome_about_new (_("Mergeant"), VERSION,
-					  "(C) 1999-2001 Vivien Malerba",
+					  "(C) 1999-2002 Vivien Malerba",
 					  _("A Database admin tool for any SQL database "
 					   "accessible with the gnome-db module."),
 					  authors,
