@@ -256,7 +256,8 @@ data_form_new_edit (QueryExec *qx, gulong numrow)
 	i = 0;
 	while (list) {
 		QueryView *qv = QUERY_VIEW (list->data);
-		if (IS_DB_TABLE (qv->obj) && (DB_TABLE (qv->obj) == query_exec_get_modif_table (qx))) {
+		if (IS_DB_TABLE (query_view_get_view_obj (qv)) && 
+		    (DB_TABLE (query_view_get_view_obj (qv)) == query_exec_get_modif_table (qx))) {
 			form->priv->qv = qv;
 			i++;
 		}
@@ -294,7 +295,8 @@ data_form_new_insert (QueryExec *qx)
 	i = 0;
 	while (list) {
 		QueryView *qv = QUERY_VIEW (list->data);
-		if (IS_DB_TABLE (qv->obj) && (DB_TABLE (qv->obj) == query_exec_get_modif_table (qx))) {
+		if (IS_DB_TABLE (query_view_get_view_obj (qv)) && 
+		    (DB_TABLE (query_view_get_view_obj (qv)) == query_exec_get_modif_table (qx))) {
 			form->priv->qv = qv;
 			i++;
 		}
@@ -341,7 +343,7 @@ data_form_new_select (QueryExec *qx, QueryView *qv, gulong numrow)
 
 	g_return_val_if_fail (qx && IS_QUERY_EXEC (qx), NULL);
 	g_return_val_if_fail (qv && IS_QUERY_VIEW (qv), NULL);
-	g_return_val_if_fail (QUERY_VIEW (qv)->query != query_exec_get_query_ref (qx), NULL);
+	g_return_val_if_fail (query_view_get_query (QUERY_VIEW (qv)) != query_exec_get_query_ref (qx), NULL);
 
 	/* FIXME: how to set the parameters for the GtkVBox? */
 	obj = g_object_new (DATA_FORM_TYPE, NULL);
@@ -491,12 +493,12 @@ build_edit_insert_form_structure (DataForm * form)
 	while (list) {
 		qf = QUERY_FIELD (list->data);
 
-		if (qf->field_type == QUERY_FIELD_FIELD) {
+		if (query_field_get_ftype (qf) == QUERY_FIELD_FIELD) {
 			QueryView *qv;
 
 			qv = query_field_field_get_queryview (qf);
-			if (qv && IS_DB_TABLE (qv->obj) && 
-			    (DB_TABLE (qv->obj) == query_exec_get_modif_table (form->priv->qx))) {
+			if (qv && IS_DB_TABLE (query_view_get_view_obj (qv)) && 
+			    (DB_TABLE (query_view_get_view_obj (qv)) == query_exec_get_modif_table (form->priv->qx))) {
 				GSList *joins;
 				QueryJoin *qj = NULL;
 				
@@ -511,7 +513,7 @@ build_edit_insert_form_structure (DataForm * form)
 					g_slist_free (joins);
 				}
 				
-				if (qj) { /* We create a sub form a QueryJoin */
+				if (qj) { /* We create a sub form for the QueryJoin */
 					QueryView *oqv;
 					QueryField *oqf = NULL;
 					DbField *ofield = NULL;
@@ -522,16 +524,18 @@ build_edit_insert_form_structure (DataForm * form)
 					gint pos;
 
 					/* set the other QueryView */
-					if (qj->ant_view == qv)
-						oqv = qj->suc_view;
+					if (query_join_get_ant_view (qj) == qv)
+						oqv = query_join_get_suc_view (qj);
 					else
-						oqv = qj->ant_view;
+						oqv = query_join_get_ant_view (qj);
 					
 					/* set the other DbField */
-					if (QUERY_JOIN_PAIR (qj->pairs->data)->ant_field == G_OBJECT (qf))
-						ofield = DB_FIELD (QUERY_JOIN_PAIR (qj->pairs->data)->suc_field);
-					else
-						ofield = DB_FIELD (QUERY_JOIN_PAIR (qj->pairs->data)->ant_field);
+					/* FIXME */
+					/* if (QUERY_JOIN_PAIR (query_join_get_pairs (qj)->data)->ant_field ==  */
+/* 					    G_OBJECT (qf)) */
+/* 						ofield = DB_FIELD (QUERY_JOIN_PAIR (query_join_get_pairs (qj)->data)->suc_field); */
+/* 					else */
+/* 						ofield = DB_FIELD (QUERY_JOIN_PAIR (query_join_get_pairs (qj)->data)->ant_field); */
 
 					/* set the other QueryField */
 					oqf = query_get_field_by_dbfield (query_exec_get_query_ref (form->priv->qx), 
@@ -572,7 +576,8 @@ build_edit_insert_form_structure (DataForm * form)
 					ee->value_ref = vr;
 				}
 				else { /* No QueryJoin for that QueryField */
-					if (qf->is_printed && !qf->is_hidden) {
+					if (query_field_get_is_printed (qf) && 
+					    !query_field_get_is_hidden (qf)) {
 						DataEntry *de;
 						DataHandler *dh;
 						const GdaValue *value;
@@ -806,8 +811,8 @@ build_complete_noglade_form_widget (DataForm * form)
 		gtk_table_attach_defaults (GTK_TABLE (topwid), wid,
 					   1, 2, i, i+1);
 		
-		if (ENTRY_ENTITY (list->data)->qf->name)
-			str = g_strdup_printf ("%s:", ENTRY_ENTITY (list->data)->qf->name);
+		if (query_field_get_name (ENTRY_ENTITY (list->data)->qf))
+			str = g_strdup_printf ("%s:", query_field_get_name (ENTRY_ENTITY (list->data)->qf));
 		else
 			str = g_strdup_printf ("%s:", DB_FIELD (ENTRY_ENTITY (list->data)->value_ref->ref)->name);
 
@@ -1174,7 +1179,7 @@ form_do_commit (DataForm *form)
 			value->dh = data_entry_get_data_handler (de);
 
 			/* QF type test */
-			if (value->field->field_type != QUERY_FIELD_FIELD)
+			if (query_field_get_ftype (value->field) != QUERY_FIELD_FIELD)
 				g_warning ("EntryEntity %p does not reference a DbField\n", list->data);
 			
 			if (data_entry_is_value_default (de)) {
@@ -1225,7 +1230,7 @@ form_do_commit (DataForm *form)
 	g_slist_free (values);
 
 	/* Query execution */
-	rs = server_access_do_query (form->priv->qv->query->conf->srv, query, 
+	rs = server_access_do_query (query_view_get_query (form->priv->qv)->conf->srv, query, 
 				     SERVER_ACCESS_QUERY_SQL);
 	if (rs) {
 		g_signal_emit (G_OBJECT (form), data_form_signals [ACTION_REFRESH], 0);
@@ -1317,14 +1322,14 @@ make_where_cond_sql_part (DataForm *form)
 		QueryField *qf;
 		qf = QUERY_FIELD (list->data);
 
-		if (qf->field_type == QUERY_FIELD_FIELD) {
+		if (query_field_get_ftype (qf) == QUERY_FIELD_FIELD) {
 			DbField *field;
 			DbTable *table;
 			gint pos;
 			const GdaValue *value;
 
 			field = query_field_field_get_db_field (qf);
-			table = database_find_table_from_field (form->priv->qv->query->conf->db, field);
+			table = database_find_table_from_field (query_view_get_query (form->priv->qv)->conf->db, field);
 			if (table == modif_table) { /* WHERE condition only on the modified table */
 				if (first_field)
 					first_field = FALSE;
@@ -1341,7 +1346,7 @@ make_where_cond_sql_part (DataForm *form)
 					gchar *str;
 					DataHandler *dh;
 					
-					dh = server_access_get_object_handler (form->priv->qv->query->conf->srv,
+					dh = server_access_get_object_handler (query_view_get_query (form->priv->qv)->conf->srv,
 									       G_OBJECT (field->type));
 					if (dh)
 						str = data_handler_get_sql_from_value (dh, value);
@@ -1448,7 +1453,7 @@ form_do_delete (DataForm *form)
 	}
 
 
-	rs = server_access_do_query (form->priv->qv->query->conf->srv, query, SERVER_ACCESS_QUERY_SQL);
+	rs = server_access_do_query (query_view_get_query (form->priv->qv)->conf->srv, query, SERVER_ACCESS_QUERY_SQL);
 	if (rs) {
 		g_signal_emit (G_OBJECT (form), data_form_signals [ACTION_REFRESH], 0);
 		g_object_unref (G_OBJECT (rs));
