@@ -25,6 +25,7 @@
 #include "relship.h"
 #include <bonobo.h>
 #include <libgnomedb/libgnomedb.h>
+#include "item-selector.h"
 #include "object-selector.h"
 #include "settings.h"
 
@@ -66,6 +67,8 @@ todo (ConfManager * conf, const gchar *file, int line)
 void
 sql_conn_open_cb (GtkWidget * widget, ConfManager * conf)
 {
+	gchar *str;
+
 	if (*(conf->srv->gda_datasource->str)) {
 		/* we check if we have a password or not; if not, ask for one, except if we 
 		   have already entered it into the gnome-db login dialog */
@@ -102,10 +105,12 @@ sql_conn_open_cb (GtkWidget * widget, ConfManager * conf)
 			gtk_widget_show (vb);
 
 			label = gtk_label_new (NULL);
-			gtk_label_set_markup (GTK_LABEL (label), 
-					      _("<span weight=\"bold\" size=\"larger\">Password requested</span>\n\n"
-						"Enter password to open the connection,\n"
-						"or leave it empty if no password is required.\n\n"));
+			str = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
+					       _("Password requested"),
+					       _("Enter password to open the connection,\n"
+						 "or leave it empty if no password is required.\n\n"));
+			gtk_label_set_markup (GTK_LABEL (label), str);
+			g_free (str);
 			gtk_box_pack_start (GTK_BOX (vb), label, TRUE, TRUE, GNOME_PAD/2.);
 			gtk_widget_show (label);
 
@@ -154,6 +159,7 @@ void
 sql_conn_close_cb (GtkWidget * widget, ConfManager * conf)
 {
 	gint answer;
+	gchar *str;
 	GtkWidget *dialog, *label, *hb, *pixmap, *vb;
 
 	/* ask if the user really wants to quit */
@@ -166,12 +172,13 @@ sql_conn_close_cb (GtkWidget * widget, ConfManager * conf)
 						      NULL);
 		
 		label = gtk_label_new (NULL);
-		gtk_label_set_markup (GTK_LABEL (label), 
-				      _("<span weight=\"bold\" size=\"larger\">Save changes before "
-					"disconnecting?</span>\n\n"
-					"Some data have not yet been saved and will be lost\n"
-					"if not saved before "
-					"closing the connection."));
+		str = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
+				       _("Save changes before disconnecting?"),
+				       _("Some data have not yet been saved and will be lost\n"
+					 "if not saved before "
+					 "closing the connection."));
+		gtk_label_set_markup (GTK_LABEL (label), str);
+		g_free (str);
 	}
 	else {
 		dialog = gtk_dialog_new_with_buttons (NULL, GTK_WINDOW (conf->app), 
@@ -180,9 +187,10 @@ sql_conn_close_cb (GtkWidget * widget, ConfManager * conf)
 						      GTK_STOCK_YES, 2, NULL);
 
 		label = gtk_label_new (NULL);
-		gtk_label_set_markup (GTK_LABEL (label), 
-				      _("<span weight=\"bold\" size=\"larger\">Do you really "
-					"want to disconnect?</span>"));
+		str = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>",
+				       _("Do you really want to disconnect?"));
+		gtk_label_set_markup (GTK_LABEL (label), str);
+		g_free (str);
 	}
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
 
@@ -224,7 +232,6 @@ sql_conn_close_cb (GtkWidget * widget, ConfManager * conf)
 	gtk_widget_destroy (dialog);
 }
 
-
 /*
  * options_config_cb
  */
@@ -234,32 +241,32 @@ static gint open_conn_idle_func (ConfManager * conf);
 void
 options_config_cb (GtkWidget * widget, ConfManager * conf)
 {
-	GtkWidget *props;
+        GtkWidget *props;
 
-	props = gnome_db_login_dialog_new (_("Connection's configuration"));
-	if (gnome_db_login_dialog_run (GNOME_DB_LOGIN_DIALOG (props))) {
-		conf->save_up_to_date = FALSE;
-		
-		/* fetching DSN and provider */
-		g_string_assign (conf->srv->gda_datasource, 
-				 gnome_db_login_dialog_get_dsn(GNOME_DB_LOGIN_DIALOG (props)));
-		g_string_assign (conf->srv->user_name, 
-				 gnome_db_login_dialog_get_username(GNOME_DB_LOGIN_DIALOG (props)));
-		g_string_assign (conf->srv->password, 
-				 gnome_db_login_dialog_get_password(GNOME_DB_LOGIN_DIALOG (props)));
-		
-		if (conf->user_name) 
-			g_free (conf->user_name);
-		conf->user_name = g_strdup (conf->srv->gda_datasource->str);
-		if (conf->user_passwd) 
-			g_free (conf->user_passwd);
-		conf->user_passwd = g_strdup (conf->srv->password->str);
-		
-		/* if the connection was to be opened, we open it now */
-		if (conf->conn_open_requested)
-			gtk_idle_add ((GtkFunction) open_conn_idle_func, conf);
-	}
-	gtk_widget_destroy (props);
+        props = gnome_db_login_dialog_new (_("Connection's configuration"));
+        if (gnome_db_login_dialog_run (GNOME_DB_LOGIN_DIALOG (props))) {
+                conf->save_up_to_date = FALSE;
+
+                /* fetching DSN and provider */
+                g_string_assign (conf->srv->gda_datasource,
+                                 gnome_db_login_dialog_get_dsn(GNOME_DB_LOGIN_DIALOG (props)));
+                g_string_assign (conf->srv->user_name,
+                                 gnome_db_login_dialog_get_username(GNOME_DB_LOGIN_DIALOG (props)));
+                g_string_assign (conf->srv->password,
+                                 gnome_db_login_dialog_get_password(GNOME_DB_LOGIN_DIALOG (props)));
+
+                if (conf->user_name)
+                        g_free (conf->user_name);
+                conf->user_name = g_strdup (conf->srv->gda_datasource->str);
+                if (conf->user_passwd)
+                        g_free (conf->user_passwd);
+                conf->user_passwd = g_strdup (conf->srv->password->str);
+
+                /* if the connection was to be opened, we open it now */
+                if (conf->conn_open_requested)
+                        gtk_idle_add ((GtkFunction) open_conn_idle_func, conf);
+        }
+        gtk_widget_destroy (props);
 }
 
 
@@ -267,13 +274,11 @@ options_config_cb (GtkWidget * widget, ConfManager * conf)
 static gint
 open_conn_idle_func (ConfManager * conf)
 {
-	sql_conn_open_cb (NULL, conf);
-	return FALSE;
+        sql_conn_open_cb (NULL, conf);
+        return FALSE;
 }
 
 
-
-
 /*
  * sql_show_relations_cb
  */
@@ -776,9 +781,9 @@ config_display_plugins_cb (GObject * obj, ConfManager * conf)
 		gtk_misc_set_alignment (GTK_MISC (wid), 0, 0);
 		gtk_table_attach (GTK_TABLE (table), wid, 0, 1, 2, 3,
 				  GTK_FILL, 0, GNOME_PAD/2., GNOME_PAD/2.);
-		wid = choice_combo_new ();
+		wid = item_selector_new (ITEM_SELECTOR_MENU);
 		g_object_set_data (G_OBJECT (conf->config_plugins_dlg), "selcc", wid);
-		choice_combo_set_content (CHOICE_COMBO (wid), NULL, 0);
+		item_selector_set_static_content_offset (ITEM_SELECTOR (wid), NULL, 0);
 		gtk_table_attach_defaults (GTK_TABLE (table), wid, 1, 2, 2, 3);
 		g_signal_connect (G_OBJECT (wid), "selection_changed",
 				  G_CALLBACK (plugins_ch_combo_sel_changed_cb), conf);
@@ -846,9 +851,9 @@ config_display_plugins_cb (GObject * obj, ConfManager * conf)
 		gtk_misc_set_alignment (GTK_MISC (wid), 0, 0);
 		gtk_table_attach (GTK_TABLE (table), wid, 0, 1, 2, 3,
 				  GTK_FILL, 0, GNOME_PAD/2., GNOME_PAD/2.);
-		wid = choice_combo_new ();
+		wid = item_selector_new (ITEM_SELECTOR_MENU);
 		g_object_set_data (G_OBJECT (conf->config_plugins_dlg), "seltabplug", wid);
-		choice_combo_set_content (CHOICE_COMBO (wid), NULL, 0);
+		item_selector_set_static_content_offset (ITEM_SELECTOR (wid), NULL, 0);
 		gtk_table_attach_defaults (GTK_TABLE (table), wid, 1, 2, 2, 3);
 		g_signal_connect (G_OBJECT (wid), "selection_changed",
 				  G_CALLBACK (plugins_ch_combo_sel_changed_cb2), conf);
@@ -974,8 +979,8 @@ data_types_os_select_cb (ObjectSelector * os, ServerDataType *dt, ConfManager *
 						 G_CALLBACK (plugins_ch_combo_sel_changed_cb), 
 						 conf);
 
-		choice_combo_set_content_func (CHOICE_COMBO (wid), blist, 
-					       (gchar * (*) (gpointer data)) data_handler_get_plugin_name_proxy);
+		item_selector_set_static_content_func (ITEM_SELECTOR (wid), blist, 
+						       (ItemSelectorFunc) data_handler_get_plugin_name_proxy);
 		/* if a plugin is already bound, set it as default */
 		dh = server_access_get_object_handler (conf->srv, G_OBJECT (dt));
 		if (dh) {
@@ -986,12 +991,12 @@ data_types_os_select_cb (ObjectSelector * os, ServerDataType *dt, ConfManager *
 
 			i = g_slist_index (blist, dh);
 			if (i >= 0) {
-				choice_combo_set_selection_num (CHOICE_COMBO (wid), i);
+				item_selector_set_selection_num (ITEM_SELECTOR (wid), i);
 				/* updating the plugin description */
 				plugin_update_description (w1, w2, dh);
 			}
 			else {
-				choice_combo_set_selection_num (CHOICE_COMBO (wid), 0);
+				item_selector_set_selection_num (ITEM_SELECTOR (wid), 0);
 				plugin_update_description (w1, w2, NULL);
 			}
 		}
@@ -1019,7 +1024,7 @@ data_types_os_select_cb (ObjectSelector * os, ServerDataType *dt, ConfManager *
 		/* choice combo */
 		g_object_set_data (G_OBJECT (conf->config_plugins_dlg), "dt", NULL);	
 		wid = g_object_get_data (G_OBJECT (conf->config_plugins_dlg), "selcc");
-		choice_combo_set_content (CHOICE_COMBO (wid), NULL, 0);
+		item_selector_set_static_content_offset (ITEM_SELECTOR (wid), NULL, 0);
 	}
 }
 
@@ -1027,7 +1032,10 @@ data_types_os_select_cb (ObjectSelector * os, ServerDataType *dt, ConfManager *
 static gchar *
 data_handler_get_plugin_name_proxy (DataHandler *dh)
 {
-	return g_strdup (data_handler_get_plugin_name (dh));
+	if (dh)
+		return g_strdup (data_handler_get_plugin_name (dh));
+	else 
+		return g_strdup (_("None"));
 }
 
 
@@ -1144,8 +1152,8 @@ table_field_os_select_cb (ObjectSelector * os, GObject *obj, ConfManager * conf)
 						 G_CALLBACK (plugins_ch_combo_sel_changed_cb2), 
 						 conf);
 		
-		choice_combo_set_content_func (CHOICE_COMBO (wid), blist,
-					       (gchar * (*) (gpointer data)) data_handler_get_plugin_name_proxy);
+		item_selector_set_static_content_func (ITEM_SELECTOR (wid), blist,
+						       (ItemSelectorFunc) data_handler_get_plugin_name_proxy);
 
 		/* if a plugin is already bound, set it as default */
 		dh = server_access_get_object_handler (conf->srv, G_OBJECT (field));
@@ -1156,12 +1164,12 @@ table_field_os_select_cb (ObjectSelector * os, GObject *obj, ConfManager * conf)
 
 			i = g_slist_index (blist, dh);
 			if (i >= 0) {
-				choice_combo_set_selection_num (CHOICE_COMBO (wid), i);
+				item_selector_set_selection_num (ITEM_SELECTOR (wid), i);
 				/* updating the plugin description */
 				plugin_update_description (w1, w2, dh);
 			}
 			else {
-				choice_combo_set_selection_num (CHOICE_COMBO (wid), 0);
+				item_selector_set_selection_num (ITEM_SELECTOR (wid), 0);
 				plugin_update_description (w1, w2, NULL);
 			}
 		}
@@ -1190,7 +1198,7 @@ table_field_os_select_cb (ObjectSelector * os, GObject *obj, ConfManager * conf)
 		/* choice combo */
 		g_object_set_data (G_OBJECT (conf->config_plugins_dlg), "obj", NULL);
 		wid = g_object_get_data (G_OBJECT (conf->config_plugins_dlg), "seltabplug");
-		choice_combo_set_content (CHOICE_COMBO (wid), NULL, 0);
+		item_selector_set_static_content_offset (ITEM_SELECTOR (wid), NULL, 0);
 	}
 }
 
@@ -1633,6 +1641,7 @@ void
 quit_cb (GtkWidget * widget, ConfManager * conf)
 {
 	gint answer;
+	gchar *str;
 	GtkWidget *dialog, *label, *hb, *vb, *pixmap;
 
 	/* ask if the user really wants to quit */
@@ -1644,11 +1653,12 @@ quit_cb (GtkWidget * widget, ConfManager * conf)
 						      NULL);
 
 		label = gtk_label_new (NULL);
-		gtk_label_set_markup (GTK_LABEL (label), 
-				      _("<span weight=\"bold\" size=\"larger\">Do you want to save "
-					"before quitting?</span>\n\n"
-					"Some data have not yet been saved and will be lost\n"
-					"if not saved before quitting."));
+		str = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
+				       _("Do you want to save before quitting?"),
+				       _("Some data have not yet been saved and will be lost\n"
+					 "if not saved before quitting."));
+		gtk_label_set_markup (GTK_LABEL (label), str);
+		g_free (str);
 
 	}
 	else {
@@ -1657,9 +1667,10 @@ quit_cb (GtkWidget * widget, ConfManager * conf)
 						      GTK_STOCK_YES, 2, NULL);
 
 		label = gtk_label_new (NULL);
-		gtk_label_set_markup (GTK_LABEL (label), 
-				      _("<span weight=\"bold\" size=\"larger\">Do you really want "
-					"to quit?</span>"));
+		str = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>",
+				       _("Do you really want to quit?"));
+		gtk_label_set_markup (GTK_LABEL (label), str);
+		g_free (str);
 	}
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
 
@@ -1704,9 +1715,7 @@ static void
 quit_real (ConfManager * conf)
 {
 	/* saving state of the configuration */
-	gnome_db_config_set_string ("/apps/Mergeant/SqlServer/datasource",
-				    conf->srv->gda_datasource->str);
-	gnome_db_config_set_string ("/apps/Mergeant/SqlServer/plugins_dir", conf->plugins_dir);
+	settings_set_plugins_dir (conf->plugins_dir);
 
 	if (!conf->save_up_to_date) /* ALL data will be lost */
 		conf->save_up_to_date = TRUE;
@@ -1879,6 +1888,7 @@ sql_server_conn_open_cb (GObject * wid, ConfManager * conf)
 			/* ask to refresh the structure */
 			GtkWidget *dlg, *label, *hb, *pixmap;
 			gint answer;
+			gchar *str;
 
 			dlg = gtk_dialog_new_with_buttons (NULL, GTK_WINDOW (conf->app),
 							   0,
@@ -1887,16 +1897,16 @@ sql_server_conn_open_cb (GObject * wid, ConfManager * conf)
 			
 
 			label = gtk_label_new (NULL);
-			gtk_label_set_markup (GTK_LABEL (label), 
-					      _("<span weight=\"bold\" size=\"larger\">"
-						"Synchronize with the database structure?</span>\n\n"
-						"To update the list of data types,"
-						" tables, functions, "
-						"sequences, ...\nfrom the server (recommanded "
-						"if the structure of the DB has been "
-						"modified).\n"
-						"Say Yes if unsure."));
-
+			str = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
+					       _("Synchronize with the database structure?"),
+					       _("To update the list of data types,"
+						 " tables, functions, "
+						 "sequences, ...\nfrom the server (recommended "
+						 "if the structure of the DB has been "
+						 "modified).\n"
+						 "Say Yes if unsure."));
+			gtk_label_set_markup (GTK_LABEL (label), str);
+			g_free (str);
 
 			hb = gtk_hbox_new (FALSE, GNOME_PAD);
 			gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), hb, TRUE, TRUE, GNOME_PAD);
