@@ -467,9 +467,6 @@ view_records_button_cb (GObject   * obj, MainPageTable * mpt)
 				/* QueryField */
 				qf = query_field_allfields_dbtable_new (rdata->query, rdata->table->name,
 									NULL, G_OBJECT (rdata->table));
-				qf = QUERY_FIELD (query_field_new (rdata->query, rdata->table->name, 
-								   QUERY_FIELD_ALLFIELDS));
-				
 				query_field_set_is_printed (qf, TRUE);
 				query_add_field (QUERY (rdata->query), qf);
 				
