@@ -68,7 +68,7 @@ prepare_app (ConfManager * conf)
 
 
 	/* getting the previously saved parameters */
-	str = gnome_db_config_get_string ("/apps/Mergeant/SqlServer/plugins_dir");
+	str = settings_get_plugins_dir ();
 	if (str)
 		conf->plugins_dir = str;
 	else
