@@ -26,16 +26,34 @@ static void query_cond_class_init (QueryCondClass * class);
 static void query_cond_init       (QueryCond * qc);
 static void query_cond_dispose    (GObject   * object);
 
+struct _QueryCondPrivate 
+{
+	Query           *query;
+	QueryCondType    cond_type;
+		
+	QueryCond       *parent;
+	/* sub QueryCond objects if cond_type != QUERY_COND_COND */
+	GSList          *children;
+
+	/* if cond_type == QUERY_COND_COND */
+	QueryField      *left_op;
+	QueryField      *right_op;
+	QueryField      *right_op2;
+	QueryCondOpType  op_type;
+};
+
+
 /* get a pointer to the parents to be able to call their destructor */
 static GObjectClass   *parent_class = NULL;
 
 enum
 {
 	CHANGED,
+	NULLIFIED,
 	LAST_SIGNAL
 };
 
-static gint signals[LAST_SIGNAL] = { 0 };
+static gint signals[LAST_SIGNAL] = { 0, 0 };
 
 
 guint
@@ -75,8 +93,18 @@ query_cond_class_init (QueryCondClass * class)
 			      NULL, NULL,
 			      marshal_VOID__VOID, G_TYPE_NONE,
 			      0);
+	signals[NULLIFIED] =
+		g_signal_new ("nullified",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (QueryCondClass, nullified),
+			      NULL, NULL,
+			      marshal_VOID__VOID, G_TYPE_NONE,
+			      0);
 
 	class->changed = NULL;
+	class->nullified = NULL;
+
 	object_class->dispose = query_cond_dispose;
 }
 
@@ -84,13 +112,15 @@ query_cond_class_init (QueryCondClass * class)
 static void
 query_cond_init (QueryCond * qc)
 {
-	qc->query = NULL;
-	qc->cond_type = QUERY_COND_AND;
-	qc->children = NULL;
-	qc->op_type = QUERY_COND_OP_EQUAL;
-	qc->left_op = NULL;
-	qc->right_op = NULL;
-	qc->right_op2 = NULL;
+	qc->priv = g_new0 (QueryCondPrivate, 1);
+	qc->priv->query = NULL;
+	qc->priv->cond_type = QUERY_COND_AND;
+	qc->priv->parent = NULL;
+	qc->priv->children = NULL;
+	qc->priv->op_type = QUERY_COND_OP_EQUAL;
+	qc->priv->left_op = NULL;
+	qc->priv->right_op = NULL;
+	qc->priv->right_op2 = NULL;
 }
 
 
@@ -102,24 +132,24 @@ query_cond_new_as_node (Query *q, QueryCondType type)
 	QueryCond *qc;
 
 	g_return_val_if_fail (q && IS_QUERY (q), NULL);
-	g_return_val_if_fail (type == QUERY_COND_COND, NULL);
+	g_return_val_if_fail (type != QUERY_COND_COND, NULL);
 
 	obj = g_object_new (QUERY_COND_TYPE, NULL);
 	qc = QUERY_COND (obj);
 
-	qc->query = q;
-	qc->cond_type = type;
+	qc->priv->query = q;
+	qc->priv->cond_type = type;
 
 	return obj;
 }
 
 
-static void query_field_destroy_notify (QueryCond *qc, QueryField *qf);
+static void query_field_nullified_cb (QueryField *qf, QueryCond *qc);
 GObject *
 query_cond_new_as_leaf      (Query *q, QueryCondOpType type, 
 			     QueryField *left_op, QueryField *right_op, QueryField *right_op2)
 {
-GObject   *obj;
+	GObject   *obj;
 	QueryCond *qc;
 
 	g_return_val_if_fail (q && IS_QUERY (q), NULL);
@@ -127,25 +157,28 @@ GObject   *obj;
 	obj = g_object_new (QUERY_COND_TYPE, NULL);
 	qc = QUERY_COND (obj);
 
-	qc->query = q;
-	qc->cond_type = QUERY_COND_COND;
-	qc->op_type = type;
+	qc->priv->query = q;
+	qc->priv->cond_type = QUERY_COND_COND;
+	qc->priv->op_type = type;
 
-	if (IS_QUERY_FIELD (left_op) && (QUERY_FIELD (left_op)->query == q)) {
-		qc->left_op = left_op;
-		g_object_weak_ref (G_OBJECT (left_op), (GWeakNotify) query_field_destroy_notify, qc);
+	if (IS_QUERY_FIELD (left_op) && (query_field_get_query (QUERY_FIELD (left_op)) == q)) {
+		qc->priv->left_op = left_op;
+		g_signal_connect (G_OBJECT (left_op), "nullified",
+				  G_CALLBACK (query_field_nullified_cb), qc);
 	}
-	if (IS_QUERY_FIELD (right_op) && (QUERY_FIELD (right_op)->query == q)) {
-		qc->right_op = right_op;
-		g_object_weak_ref (G_OBJECT (right_op), (GWeakNotify) query_field_destroy_notify, qc);
+	if (IS_QUERY_FIELD (right_op) && (query_field_get_query (QUERY_FIELD (right_op)) == q)) {
+		qc->priv->right_op = right_op;
+		g_signal_connect (G_OBJECT (right_op), "nullified",
+				  G_CALLBACK (query_field_nullified_cb), qc);
 	}
 
 	switch (type) {
-	case QUERY_COND_OP_IN:
 	case QUERY_COND_OP_BETWEEN:
-		if (IS_QUERY_FIELD (right_op2) && (QUERY_FIELD (right_op2)->query == q)) {
-			qc->right_op2 = right_op2;
-			g_object_weak_ref (G_OBJECT (right_op2), (GWeakNotify) query_field_destroy_notify, qc);
+		if (IS_QUERY_FIELD (right_op2) && (query_field_get_query (QUERY_FIELD (right_op2)) == q)) {
+			qc->priv->right_op2 = right_op2;
+
+			g_signal_connect (G_OBJECT (right_op2), "nullified",
+					  G_CALLBACK (query_field_nullified_cb), qc);
 		}
 		break;
 	default:
@@ -155,17 +188,11 @@ GObject   *obj;
 	return obj;	
 }
 
-
+static void query_cond_nullify (GObject *object, gboolean signal_it);
 static void 
-query_field_destroy_notify (QueryCond *qc, QueryField *qf)
+query_field_nullified_cb (QueryField *qf, QueryCond *qc)
 {
-	if (qc->left_op == qf)
-		qc->left_op = NULL;
-	if (qc->right_op == qf)
-		qc->right_op = NULL;
-	if (qc->right_op2 == qf)
-		qc->right_op2 = NULL;
-	query_cond_free (qc);
+	query_cond_nullify (G_OBJECT (qc), TRUE);
 }
 
 GObject   *
@@ -174,14 +201,14 @@ query_cond_new_copy (QueryCond *qc)
 	QueryCond *nqc;
 
 	g_return_val_if_fail (qc && IS_QUERY_COND (qc), NULL);
-	if (qc->cond_type == QUERY_COND_COND) 
-		nqc = QUERY_COND (query_cond_new_as_leaf (qc->query, qc->op_type, 
-							  qc->left_op, qc->right_op, qc->right_op2));
+	if (qc->priv->cond_type == QUERY_COND_COND) 
+		nqc = QUERY_COND (query_cond_new_as_leaf (qc->priv->query, qc->priv->op_type, 
+							  qc->priv->left_op, qc->priv->right_op, qc->priv->right_op2));
 	else {
 		GSList *list;
 
-		nqc = QUERY_COND (query_cond_new_as_node (qc->query, qc->cond_type));
-		list = qc->children;
+		nqc = QUERY_COND (query_cond_new_as_node (qc->priv->query, qc->priv->cond_type));
+		list = qc->priv->children;
 		while (list) {
 			QueryCond *nchild;
 
@@ -190,104 +217,100 @@ query_cond_new_copy (QueryCond *qc)
 			list = g_slist_next (list);
 		}
 	}
+	nqc->priv->parent = qc->priv->parent;
 	
 	return G_OBJECT (nqc);
 }
 
-void
-query_cond_free (QueryCond *qc)
-{
-	g_return_if_fail (qc && IS_QUERY_COND (qc));
-
-	g_object_unref (G_OBJECT (qc));
-}
-
-
-static void query_cond_reinit (GObject   * object);
 
 static void
-query_cond_dispose (GObject   * object)
+query_cond_dispose (GObject *object)
 {
 	g_return_if_fail (object && IS_QUERY_COND (object));
 	
-	query_cond_reinit (object);
+	g_print ("query_cond_dispose (%p)\n", object); /* PRINT */
+	if (QUERY_COND (object)->priv) {
+		query_cond_nullify (object, FALSE);
+		g_free (QUERY_COND (object)->priv);
+		QUERY_COND (object)->priv = NULL;
+	}
+	
 	/* for the parent class */
 	parent_class->dispose (object);
 }
 
-static void child_changed_cb (QueryCond *child, QueryCond *qc);
-static void child_node_destroy_notify (QueryCond *qc, QueryCond *child); /* GWeakNotify */
+static void child_nullified_cb (QueryCond *child, QueryCond *qc);
 static void 
-query_cond_reinit (GObject   * object)
+query_cond_nullify (GObject *object, gboolean signal_it)
 {
 	QueryCond *qc;
-	GSList *list;
 
 	g_return_if_fail (object && IS_QUERY_COND (object));
 
-
 	qc = QUERY_COND (object);
 
-	/* operators */
-	if (qc->left_op) {
-		g_object_weak_unref (G_OBJECT (qc->left_op), (GWeakNotify) query_field_destroy_notify, qc);
-		qc->left_op = NULL;
+	if (qc->priv->cond_type == QUERY_COND_COND) {
+		/* operators */
+		if (qc->priv->left_op) {
+			g_signal_handlers_disconnect_by_func (G_OBJECT (qc->priv->left_op),
+							      G_CALLBACK (query_field_nullified_cb), qc);
+			qc->priv->left_op = NULL;
+		}
+		
+		if (qc->priv->right_op) {
+			g_signal_handlers_disconnect_by_func (G_OBJECT (qc->priv->right_op),
+							      G_CALLBACK (query_field_nullified_cb), qc);
+			qc->priv->right_op = NULL;
+		}
+		
+		if (qc->priv->right_op2) {
+			g_signal_handlers_disconnect_by_func (G_OBJECT (qc->priv->right_op2),
+							      G_CALLBACK (query_field_nullified_cb), qc);
+			qc->priv->right_op2 = NULL;
+		}
 	}
-
-	if (qc->right_op) {
-		g_object_weak_unref (G_OBJECT (qc->right_op), (GWeakNotify) query_field_destroy_notify, qc);
-		qc->right_op = NULL;
+	else {
+		/* children if any */
+		while (qc->priv->children) 
+			child_nullified_cb (QUERY_COND (qc->priv->children->data), qc);
 	}
+	qc->priv->parent = NULL;
 
-	if (qc->right_op2) {
-		g_object_weak_unref (G_OBJECT (qc->right_op2), (GWeakNotify) query_field_destroy_notify, qc);
-		qc->right_op2 = NULL;
-	}				     
-
-	/* children if any */
-	list = qc->children;
-	while (list) {
-		g_object_weak_unref (G_OBJECT (list->data), (GWeakNotify) child_node_destroy_notify, qc);
-		g_signal_handlers_disconnect_by_func (G_OBJECT (list->data), G_CALLBACK (child_changed_cb), qc);
-		query_cond_free (QUERY_COND (list->data));
-
-		list = g_slist_next (list);
+	if (signal_it) {
+#ifdef debug_signal
+		g_print (">> 'NULLIFIED' from query_cond_nullify\n");
+#endif
+		g_signal_emit (G_OBJECT (qc), signals[NULLIFIED], 0);
+#ifdef debug_signal
+		g_print ("<< 'NULLIFIED' from query_cond_nullify\n");
+#endif
 	}
-
-	if (qc->children) {
-		g_slist_free (qc->children);
-		qc->children = NULL;
-	}
-
-	query_cond_init (qc);
 }
 
-/* If qc node is a in fact a leaf type (=> QUERY_COND_COND) then we substitute the leaf
-   with a new AND node and we add the old leaf to that new ADD node 
-   pos == -1 => the new QueryCond must be appended to the other ones */
+static void child_changed_cb (QueryCond *child, QueryCond *qc);
+
+/* pos == -1 => the new QueryCond must be appended to the other ones */
 void
 query_cond_add_to_node (QueryCond *qc, QueryCond *child, gint pos)
 {
 	g_return_if_fail (qc && IS_QUERY_COND (qc));
 	g_return_if_fail (child && IS_QUERY_COND (child));
-
-	if (qc->cond_type == QUERY_COND_COND) {
-		/* we first need to convert to an AND node */
-		Query *q = qc->query;
-		QueryCond *qc2 = QUERY_COND (query_cond_new_copy (qc));
-		
-		query_cond_reinit (G_OBJECT (qc));
-		qc->query = q;
-		qc->cond_type = QUERY_COND_AND;
-		query_cond_add_to_node (qc, qc2, -1);
-	}
-
+	g_return_if_fail (child != qc);
+	g_return_if_fail (qc->priv->cond_type != QUERY_COND_COND);
+	g_return_if_fail (!g_slist_find (qc->priv->children, child));
+	g_return_if_fail (! child->priv->parent);
+	
 	/* add the child */
-	qc->children = g_slist_insert (qc->children, child, pos);
-	g_object_weak_ref (G_OBJECT (child), (GWeakNotify) child_node_destroy_notify, qc);
+	qc->priv->children = g_slist_insert (qc->priv->children, child, pos);
+	child->priv->parent = qc;
+	
+	g_signal_connect (G_OBJECT (child), "nullified",
+			  G_CALLBACK (child_nullified_cb), qc);
 	g_signal_connect (G_OBJECT (child), "changed",
 			  G_CALLBACK (child_changed_cb), qc);
 
+	g_object_ref (G_OBJECT (child));
+
 	/* emit signal */
 #ifdef debug_signal
 	g_print (">> 'CHANGED' from query_cond_add_to_node\n");
@@ -298,6 +321,66 @@ query_cond_add_to_node (QueryCond *qc, QueryCond *child, gint pos)
 #endif
 }
 
+static void query_cond_clean_cond (QueryCond *cond);
+static void
+child_nullified_cb (QueryCond *child, QueryCond *qc)
+{
+	qc->priv->children = g_slist_remove (qc->priv->children, child);
+	g_signal_handlers_disconnect_by_func (G_OBJECT (child),
+					      G_CALLBACK (child_nullified_cb), qc);
+	g_signal_handlers_disconnect_by_func (G_OBJECT (child),
+					      G_CALLBACK (child_changed_cb), qc);
+	child->priv->parent = NULL;
+
+	/* Clean the Querycond for extra AND/OR nodes */
+	if (qc->priv->parent)
+		query_cond_clean_cond (qc->priv->parent);
+
+	/* emit signal */
+	child_changed_cb (child, qc);
+
+	g_object_unref (G_OBJECT (child));
+}
+
+static void
+query_cond_clean_cond (QueryCond *cond)
+{
+	if (cond->priv->cond_type != QUERY_COND_COND) {
+		/* optionnaly remove any node child if that node child is a AND/OR node and
+		   only has itself 1 chil */
+		GSList *children;
+
+		children = cond->priv->children;
+		while (children) {
+			QueryCond *qc = QUERY_COND (children->data);
+			if ((qc->priv->cond_type == QUERY_COND_AND) ||
+			    (qc->priv->cond_type == QUERY_COND_OR)) {
+				/* if we have no child then we simply remove that qc */
+				if (!qc->priv->children) {
+					query_cond_nullify (G_OBJECT (qc), TRUE);
+					children = cond->priv->children;
+				}
+				else {
+					if (g_slist_length (qc->priv->children) == 1) {
+						/* move qc's child a step upper */
+						QueryCond *savechild = QUERY_COND (qc->priv->children->data);
+						g_object_ref (G_OBJECT (savechild));
+						query_cond_del_from_node (qc, savechild);
+						query_cond_add_to_node (cond, savechild, 
+									g_slist_position (cond->priv->children, children));
+						g_object_unref (G_OBJECT (savechild));
+						query_cond_del_from_node (cond, qc);
+					}
+					else
+						children = g_slist_next (children);
+				}
+			}
+			else
+				children = g_slist_next (children);
+		}
+	}
+}
+
 static void
 child_changed_cb (QueryCond *child, QueryCond *qc)
 {
@@ -310,45 +393,16 @@ child_changed_cb (QueryCond *child, QueryCond *qc)
 #endif
 }
 
-static void
-child_node_destroy_notify (QueryCond *qc, QueryCond *child) /* GWeakNotify */
-{
-	qc->children = g_slist_remove (qc->children, child);
-	g_signal_handlers_disconnect_by_func (G_OBJECT (child), G_CALLBACK (child_changed_cb), qc);
-
-	/* emit signal */
-#ifdef debug_signal
-	g_print (">> 'CHANGED' from QueryCond::child_node_destroy_notify\n");
-#endif
-	g_signal_emit (G_OBJECT (qc), signals[CHANGED], 0);
-#ifdef debug_signal
-	g_print ("<< 'CHANGED' from QueryCond::child_node_destroy_notify\n");
-#endif
-}
 
 void
-query_cond_del_from_node (QueryCond *qc, QueryCond *child, gboolean destroy_child)
+query_cond_del_from_node (QueryCond *qc, QueryCond *child)
 {
 	g_return_if_fail (qc && IS_QUERY_COND (qc));
 	g_return_if_fail (child && IS_QUERY_COND (child));
-	g_return_if_fail (child->query != qc->query);
-	g_return_if_fail (g_slist_find (qc->children, child));
+	g_return_if_fail (child->priv->query == qc->priv->query);
+	g_return_if_fail (g_slist_find (qc->priv->children, child));
 	
-	g_object_weak_unref (G_OBJECT (child), (GWeakNotify) child_node_destroy_notify, qc);
-	qc->children = g_slist_remove (qc->children, child);
-	g_signal_handlers_disconnect_by_func (G_OBJECT (child), G_CALLBACK (child_changed_cb), qc);
-
-	/* emit signal */
-#ifdef debug_signal
-	g_print (">> 'CHANGED' from query_cond_del_from_node\n");
-#endif
-	g_signal_emit (G_OBJECT (qc), signals[CHANGED], 0);
-#ifdef debug_signal
-	g_print ("<< 'CHANGED' from query_cond_del_from_node\n");
-#endif
-
-	if (destroy_child) 
-		query_cond_free (child);
+	child_nullified_cb (child, qc);
 }
 
 void
@@ -356,8 +410,8 @@ query_cond_set_leaf_type (QueryCond *qc, QueryCondOpType type)
 {
 	g_return_if_fail (qc && IS_QUERY_COND (qc));
 
-	if (qc->op_type != type) {
-		qc->op_type = type;
+	if (qc->priv->op_type != type) {
+		qc->priv->op_type = type;
 
 		/* emit signal */
 #ifdef debug_signal
@@ -391,36 +445,36 @@ query_cond_save_to_xml (QueryCond *qc)
 	node = xmlNewNode (NULL, "QueryCond");
 
 	/* Condition */
-	cstr = query_cond_get_type_str (qc->cond_type);
+	cstr = query_cond_get_type_str (qc->priv->cond_type);
 	xmlSetProp (node, "type", cstr);
 
-	if (qc->cond_type == QUERY_COND_COND) {
-		cstr = query_cond_get_op_str (qc->op_type);
+	if (qc->priv->cond_type == QUERY_COND_COND) {
+		cstr = query_cond_get_op_str (qc->priv->op_type);
 		xmlSetProp (node, "op", cstr);
 
 		/* operators */
-		if (qc->left_op) {
-			str = query_field_get_xml_id (qc->left_op);
+		if (qc->priv->left_op) {
+			str = query_field_get_xml_id (qc->priv->left_op);
 			xmlSetProp (node, "left_op", str);
 			g_free (str);
 		}
 		
-		if (qc->right_op) {
-			str = query_field_get_xml_id (qc->right_op);
+		if (qc->priv->right_op) {
+			str = query_field_get_xml_id (qc->priv->right_op);
 			xmlSetProp (node, "right_op", str);
 			g_free (str);
 		}
 		
-		if (qc->right_op2) {
-			str = query_field_get_xml_id (qc->right_op2);
+		if (qc->priv->right_op2) {
+			str = query_field_get_xml_id (qc->priv->right_op2);
 			xmlSetProp (node, "right_op2", str);
 			g_free (str);
 		}
 	}
 	else {
 		/* Children nodes? */
-		if (qc->cond_type != QUERY_COND_COND) {
-			GSList *list = qc->children;
+		if (qc->priv->cond_type != QUERY_COND_COND) {
+			GSList *list = qc->priv->children;
 			
 			while (list) {
 				xmlNodePtr nde;
@@ -448,20 +502,20 @@ query_cond_new_from_xml (ConfManager *conf, Query *q,  xmlNodePtr node)
 	obj = g_object_new (QUERY_COND_TYPE, NULL);
 	qc = QUERY_COND (obj);
 
-	qc->query = q;
+	qc->priv->query = q;
 		
 	str = xmlGetProp (node, "type");
 	if (str) {
-		qc->cond_type = query_cond_get_type_enum (str);
+		qc->priv->cond_type = query_cond_get_type_enum (str);
 		g_free (str);
 	}
 		
-	if (qc->cond_type == QUERY_COND_COND) {
+	if (qc->priv->cond_type == QUERY_COND_COND) {
 		QueryField *qf;
 
 		str = xmlGetProp (node, "op");
 		if (str) {
-			qc->op_type = query_cond_get_op_enum (str);
+			qc->priv->op_type = query_cond_get_op_enum (str);
 			g_free (str);
 		}
 			
@@ -470,7 +524,7 @@ query_cond_new_from_xml (ConfManager *conf, Query *q,  xmlNodePtr node)
 		if (str) {
 			qf = query_get_field_by_xmlid (q, str);
 			if (qf)
-				qc->left_op = qf;
+				qc->priv->left_op = qf;
 			g_free (str);
 		}
 
@@ -478,7 +532,7 @@ query_cond_new_from_xml (ConfManager *conf, Query *q,  xmlNodePtr node)
 		if (str) {
 			qf = query_get_field_by_xmlid (q, str);
 			if (qf)
-				qc->right_op = qf;
+				qc->priv->right_op = qf;
 			g_free (str);
 		}
 
@@ -486,7 +540,7 @@ query_cond_new_from_xml (ConfManager *conf, Query *q,  xmlNodePtr node)
 		if (str) {
 			qf = query_get_field_by_xmlid (q, str);
 			if (qf)
-				qc->right_op2 = qf;
+				qc->priv->right_op2 = qf;
 			g_free (str);
 		}
 	}
@@ -715,21 +769,22 @@ query_cond_render_as_sql    (QueryCond *qc, GSList * missing_values)
 	
 	string = g_string_new ("");
 
-	switch (qc->cond_type) {
+	switch (qc->priv->cond_type) {
 	case QUERY_COND_COND:
-		str = query_field_render_as_sql (qc->left_op, missing_values);
-		str2 = query_field_render_as_sql (qc->right_op, missing_values);
-		if (qc->right_op2)
-			str3 = query_field_render_as_sql (qc->right_op2, missing_values);
+		str = query_field_render_as_sql (qc->priv->left_op, missing_values);
+		str2 = query_field_render_as_sql (qc->priv->right_op, missing_values);
+		if (qc->priv->right_op2)
+			str3 = query_field_render_as_sql (qc->priv->right_op2, missing_values);
 		else
 			str3 = NULL;
 
-		switch (qc->op_type) {
+		switch (qc->priv->op_type) {
 		case QUERY_COND_OP_BETWEEN:
 			g_string_append_printf (string, "%s BETWEEN %s AND %s", str, str2, str3);
 			break;
 		default:
-			g_string_append_printf (string, "%s %s %s", str, query_cond_get_op_sql (qc->op_type) ,str2);
+			g_string_append_printf (string, "%s %s %s", str, 
+						query_cond_get_op_sql (qc->priv->op_type) ,str2);
 			break;
 		}
 		g_free (str);
@@ -738,8 +793,8 @@ query_cond_render_as_sql    (QueryCond *qc, GSList * missing_values)
 			g_free (str3);
 		break;
 	case QUERY_COND_NOT:
-		if (qc->children)
-			str = query_cond_render_as_sql (qc->children->data, missing_values);
+		if (qc->priv->children)
+			str = query_cond_render_as_sql (qc->priv->children->data, missing_values);
 		else
 			str = g_strdup ("???");
 
@@ -749,14 +804,15 @@ query_cond_render_as_sql    (QueryCond *qc, GSList * missing_values)
 		break;
 	default:
 		g_string_append (string, "(");
-		list = qc->children;
+		list = qc->priv->children;
 		while (list) {
-			str = query_cond_render_as_sql (qc->children->data, missing_values);
+			str = query_cond_render_as_sql (QUERY_COND (list->data), missing_values);
 
-			if (list == qc->children)
+			if (list == qc->priv->children)
 				g_string_append_printf (string, "%s", str);
 			else
-				g_string_append_printf (string, "%s %s", query_cond_get_type_str (qc->cond_type), str);
+				g_string_append_printf (string, " %s %s", 
+							query_cond_get_type_str (qc->priv->cond_type), str);
 
 			g_free (str);
 			list = g_slist_next (list);
@@ -794,16 +850,16 @@ query_cond_is_ancestor (QueryCond *qc, QueryCond *ancestor)
 
 	g_return_val_if_fail (qc && IS_QUERY_COND (qc), FALSE);
 	g_return_val_if_fail (ancestor && IS_QUERY_COND (ancestor), FALSE);
-	g_return_val_if_fail (ancestor->query == qc->query, FALSE);
+	g_return_val_if_fail (ancestor->priv->query == qc->priv->query, FALSE);
 
 	if (qc == ancestor)
 		return TRUE;
 
-	if (ancestor->children) {
-		if (g_slist_find (ancestor->children, qc))
+	if (ancestor->priv->children) {
+		if (g_slist_find (ancestor->priv->children, qc))
 			found = TRUE;
 		else {
-			GSList *list = ancestor->children;
+			GSList *list = ancestor->priv->children;
 			while (list && !found) {
 				found = query_cond_is_ancestor (qc, QUERY_COND (list->data));
 				list = g_slist_next (list);
@@ -815,27 +871,107 @@ query_cond_is_ancestor (QueryCond *qc, QueryCond *ancestor)
 }
 
 QueryCond *
-query_cond_get_parent (QueryCond *qc, QueryCond *top_cond)
+query_cond_get_parent (QueryCond *qc)
 {
-	QueryCond *retval = NULL;
-
 	g_return_val_if_fail (qc && IS_QUERY_COND (qc), NULL);
-	g_return_val_if_fail (top_cond && IS_QUERY_COND (top_cond), NULL);
-	g_return_val_if_fail (top_cond->query == qc->query, NULL);
 
-	if (top_cond->children) {
-		if (g_slist_find (top_cond->children, qc))
-			retval = top_cond;
-		else {
-			GSList *list = top_cond->children;
-			while (list && !qc) {
-				retval = query_cond_get_parent (qc, QUERY_COND (list->data));
-				list = g_slist_next (list);
-			}
+	return qc->priv->parent;
+}
+
+Query *
+query_cond_get_query (QueryCond *qc)
+{
+	g_return_val_if_fail (qc && IS_QUERY_COND (qc), NULL);
+
+	return qc->priv->query;
+}
+
+GSList *
+query_cond_get_children (QueryCond *qc)
+{
+	g_return_val_if_fail (qc && IS_QUERY_COND (qc), NULL);
+
+	return qc->priv->children;
+}
+
+GSList *real_query_cond_get_query_fields (QueryCond *qc, GSList *progress);
+
+/* free the list afterwards */
+GSList *
+query_cond_get_query_fields (QueryCond *qc)
+{
+	g_return_val_if_fail (qc && IS_QUERY_COND (qc), NULL);
+
+	return real_query_cond_get_query_fields (qc, NULL);
+}
+
+GSList *
+real_query_cond_get_query_fields (QueryCond *qc, GSList *progress)
+{
+	GSList *list = progress;
+
+	if (qc->priv->cond_type == QUERY_COND_COND) {
+		if (qc->priv->left_op)
+			list = g_slist_append (list, qc->priv->left_op);
+		if (qc->priv->right_op)
+			list = g_slist_append (list, qc->priv->right_op);
+		if (qc->priv->right_op2)
+			list = g_slist_append (list, qc->priv->right_op2);
+	}
+	else {
+		GSList *children = qc->priv->children;
+
+		while (children) {
+			list = real_query_cond_get_query_fields (QUERY_COND (children->data), list);
+			children = g_slist_next (children);
 		}
 	}
 
-	return retval;
+	return list;
+}
+
+QueryCondType
+query_cond_get_cond_type (QueryCond *qc)
+{
+	g_return_val_if_fail (qc && IS_QUERY_COND (qc), QUERY_COND_AND);
+
+	return qc->priv->cond_type;
+}
+
+QueryCondOpType
+query_cond_get_op_type (QueryCond *qc)
+{
+	g_return_val_if_fail (qc && IS_QUERY_COND (qc), QUERY_COND_OP_EQUAL);
+	g_return_val_if_fail (qc->priv->cond_type == QUERY_COND_COND, QUERY_COND_OP_EQUAL);
+
+	return qc->priv->op_type;
+}
+
+QueryField *
+query_cond_get_op_left (QueryCond *qc)
+{
+	g_return_val_if_fail (qc && IS_QUERY_COND (qc), NULL);
+	g_return_val_if_fail (qc->priv->cond_type == QUERY_COND_COND, NULL);
+
+	return qc->priv->left_op;
+}
+
+QueryField *
+query_cond_get_op_right (QueryCond *qc)
+{
+	g_return_val_if_fail (qc && IS_QUERY_COND (qc), NULL);
+	g_return_val_if_fail (qc->priv->cond_type == QUERY_COND_COND, NULL);
+
+	return qc->priv->right_op;
+}
+
+QueryField *
+query_cond_get_op_right2 (QueryCond *qc)
+{
+	g_return_val_if_fail (qc && IS_QUERY_COND (qc), NULL);
+	g_return_val_if_fail (qc->priv->cond_type == QUERY_COND_COND, NULL);
+
+	return qc->priv->right_op2;
 }
 
 
@@ -852,12 +988,29 @@ query_cond_dump_contents (QueryCond *qc, guint offset)
 		str[i] = ' ';
 	str[offset] = 0;
 	
-	g_print ("%s  *" D_COL_H2 "%s" D_COL_NOR "\n", str,  query_cond_get_type_str (qc->cond_type));
+	if (qc->priv->cond_type == QUERY_COND_COND) {
+		g_print ("%s  *" D_COL_H2 "%s" D_COL_NOR " ", str, 
+			 query_cond_get_type_str (qc->priv->cond_type));
+		g_print (" leftop=%p, rightop=%p, rightop2=%p\n",
+			 qc->priv->left_op, qc->priv->right_op, qc->priv->right_op2);
+	}
+	else {
+		GSList *list = qc->priv->children;
 
-	sql = query_cond_render_as_sql (qc, NULL);
-	g_print ("%s  * renders as: %s\n", str, sql);
+		g_print ("%s  *" D_COL_H2 "%s" D_COL_NOR "\n", str, 
+			 query_cond_get_type_str (qc->priv->cond_type));
+
+		while (list) {
+			query_cond_dump_contents (QUERY_COND (list->data), offset+2);
+			list = g_slist_next (list);
+		}
+	}
+
+	/* sql = query_cond_render_as_sql (qc, NULL); */
+/* 	g_print ("%s  * renders as: %s\n", str, sql); */
+/* 	g_free (sql); */
 
-	g_free (sql);
 	g_free (str);
 }
 #endif
+
