@@ -531,6 +531,9 @@ druid_finished_cb (GnomeDruidPage *druid_page, gpointer arg1, QueryCreateDruid *
 				query_field_set_alias (qf, str);
 				g_free (str);
 				query_add_field (druid->q, qf);
+				
+				/* free the reference we have on qf */
+				g_object_unref (G_OBJECT (qf));
 			}
 			else {
 				if (IS_DB_TABLE (data)) {
