@@ -42,7 +42,8 @@ static void        q_finalize         (QueryField *qf);
 static void        q_deactivate      (QueryField *qf);
 static void        q_activate        (QueryField *qf);
 static GtkWidget * q_get_edit_widget (QueryField *qf);
-static GtkWidget * q_get_sel_widget  (QueryField *qf, GCallback callback, gpointer data);
+static GtkWidget * q_get_sel_widget  (QueryField *qf, QueryField *sel_qf, 
+				      GCallback callback, gpointer data);
 static gchar     * q_render_as_sql   (QueryField *qf, GSList * missing_values);
 static xmlNodePtr  q_render_as_xml   (QueryField *qf, GSList * missing_values);
 static gchar     * q_render_as_string(QueryField *qf, GSList * missing_values);
@@ -53,21 +54,16 @@ static gboolean    q_is_equal_to     (QueryField *qf, QueryField *other);
 static GSList    * q_get_monitored_objects (QueryField *qf);
 static void        q_replace_comp    (QueryField *qf, gint ref, GObject   *old, GObject   *new);
 
-/* Weak ref from the refering object */
-static void        q_view_obj_destroyed_wr (QueryField *qf, GObject *obj);
-
 static void query_field_allfields_set_queryview (QueryField *qf, QueryView *qv);
 
 typedef struct {
 	QueryView   *view;
 
-	gboolean     obj_in_destroy;
-
 	/* XML table name if we did not find it in the first place (like TVxxx:FIyyy) */
 	gchar       *obj_name;
 } private_data;
 
-#define QF_PRIVATE_DATA(qf) ((private_data *) qf->private_data)
+#define QF_PRIVATE_DATA(qf) ((private_data *) qf->priv->private_data)
 
 QueryFieldIface * 
 query_field_allfields_get_iface()
@@ -102,48 +98,45 @@ q_init            (QueryField *qf)
 {
 	private_data *data;
 	data = g_new0 (private_data, 1);
-	qf->private_data = (gpointer) data;
+	qf->priv->private_data = (gpointer) data;
 	QF_PRIVATE_DATA(qf)->view = NULL;
 	QF_PRIVATE_DATA(qf)->obj_name = NULL;
-	QF_PRIVATE_DATA(qf)->obj_in_destroy = FALSE;
 }
 
 static void        
 q_finalize         (QueryField *qf)
 {
 	query_field_deactivate (qf);
-	if (qf->private_data) {
+	if (qf->priv->private_data) {
 		if (QF_PRIVATE_DATA(qf)->obj_name)
 			g_free (QF_PRIVATE_DATA(qf)->obj_name);
-		g_free (qf->private_data);
-		qf->private_data = NULL;
+		g_free (qf->priv->private_data);
+		qf->priv->private_data = NULL;
 	}
 }
 
+static void view_status_changed_cb (QueryView *qv, QueryField *qf);
 static void
 q_deactivate      (QueryField *qf)
 {
-	if (! qf->activated)
+	if (! qf->priv->activated)
 		return;
 
 	/* This function disconnects any event handler from any object
 	   this QueryField wants to receive events from.
 	   Here we disconnect from the table if we are connected */
 	if (QF_PRIVATE_DATA(qf)->view) {
-		gchar *str;
-		if (! QF_PRIVATE_DATA(qf)->obj_in_destroy)
-			g_object_weak_unref (G_OBJECT (QF_PRIVATE_DATA(qf)->view), 
-					     (GWeakNotify) q_view_obj_destroyed_wr, qf);
-		else
-			QF_PRIVATE_DATA(qf)->obj_in_destroy = FALSE;
+		g_signal_handlers_disconnect_by_func (G_OBJECT (QF_PRIVATE_DATA(qf)->view),
+						      G_CALLBACK (view_status_changed_cb), qf);
 
 		if (QF_PRIVATE_DATA(qf)->obj_name) {
 			g_free (QF_PRIVATE_DATA(qf)->obj_name);
 			QF_PRIVATE_DATA(qf)->obj_name = NULL;
 		}
 
-		str = query_view_get_xml_id (QF_PRIVATE_DATA(qf)->view);
-		QF_PRIVATE_DATA(qf)->obj_name = str;
+		if (query_view_get_is_activated (QF_PRIVATE_DATA(qf)->view)) {
+			QF_PRIVATE_DATA(qf)->obj_name = query_view_get_xml_id (QF_PRIVATE_DATA(qf)->view);
+		}
 
 		QF_PRIVATE_DATA(qf)->view = NULL;
 	}
@@ -159,13 +152,13 @@ q_activate        (QueryField *qf)
 	   Here we connect to the DbTable or Query of the field we are refering to */
 	QueryView *view;
 
-	if (qf->activated)
+	if (qf->priv->activated)
 		return;
 
 	view = QF_PRIVATE_DATA(qf)->view;
 	
 	if (!view && QF_PRIVATE_DATA(qf)->obj_name) {
-		view = query_get_view_by_xmlid (qf->query, QF_PRIVATE_DATA(qf)->obj_name);
+		view = query_get_view_by_xmlid (qf->priv->query, QF_PRIVATE_DATA(qf)->obj_name);
 
 		if (view) {
 			g_free (QF_PRIVATE_DATA(qf)->obj_name);
@@ -175,8 +168,8 @@ q_activate        (QueryField *qf)
 
 	if (view) {
 		QF_PRIVATE_DATA(qf)->view = view;
-		g_object_weak_ref (G_OBJECT (QF_PRIVATE_DATA(qf)->view), 
-				   (GWeakNotify) q_view_obj_destroyed_wr, qf);
+		g_signal_connect (G_OBJECT (view), "status_changed",
+				  G_CALLBACK (view_status_changed_cb), qf);
 		query_field_set_activated (qf, TRUE);
 	}
 }
@@ -189,16 +182,16 @@ q_get_edit_widget (QueryField *qf)
 	GtkWidget *vbox, *os;
 
 	vbox = gtk_vbox_new (FALSE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), GNOME_PAD/2.);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 0);
 
-	os = object_selector_new (qf->query->conf, OBJECT_SELECTOR_QUERIES | OBJECT_SELECTOR_QVIEWS,
-				  OBJECT_SELECTOR_PARAM_TOP_QUERY, qf->query,
+	os = object_selector_new (qf->priv->query->conf, OBJECT_SELECTOR_QUERIES | OBJECT_SELECTOR_QVIEWS,
+				  OBJECT_SELECTOR_PARAM_TOP_QUERY, qf->priv->query,
 				  OBJECT_SELECTOR_PARAM_SUB_QUERIES, FALSE, -1);
 	object_selector_set_column_label (OBJECT_SELECTOR (os), 0, _("Source"));
 	object_selector_set_comments_visible (OBJECT_SELECTOR (os), FALSE);
 	if (QF_PRIVATE_DATA(qf)->view) 
 		object_selector_set_selection (OBJECT_SELECTOR (os), G_OBJECT (QF_PRIVATE_DATA(qf)->view));
-	gtk_box_pack_start (GTK_BOX (vbox), os, TRUE, TRUE, GNOME_PAD/2.);
+	gtk_box_pack_start (GTK_BOX (vbox), os, TRUE, TRUE, 0);
 	gtk_widget_show (os);
 
 	g_signal_connect (G_OBJECT (os), "selection_changed",
@@ -208,12 +201,12 @@ q_get_edit_widget (QueryField *qf)
 }
 
 static GtkWidget * 
-q_get_sel_widget (QueryField *qf, GCallback callback, gpointer data)
+q_get_sel_widget (QueryField *qf, QueryField *sel_qf, GCallback callback, gpointer data)
 {
 	GtkWidget *button;
 	gchar *str;
 
-	if (qf->activated)
+	if (qf->priv->activated)
 		str = g_strdup_printf("%s.*", query_view_get_name (QF_PRIVATE_DATA(qf)->view));
 	else
 		str = g_strdup (_("TABLE.*"));
@@ -226,6 +219,10 @@ q_get_sel_widget (QueryField *qf, GCallback callback, gpointer data)
 	   which will be transmitted when the user clicks on it */
 	g_object_set_data (G_OBJECT (button), "QF_obj_emit_sig", button);
 
+	/* state */
+	if (sel_qf == qf) 
+		gtk_widget_set_sensitive (button, FALSE);
+
 	return button;
 }
 
@@ -256,13 +253,12 @@ view_selected_cb (ObjectSelector *os, GObject *sel, QueryField *qf)
 }
 
 static void        
-q_view_obj_destroyed_wr (QueryField *qf, GObject *obj)
+view_status_changed_cb (QueryView *qv, QueryField *qf)
 {
-	QF_PRIVATE_DATA(qf)->obj_in_destroy = TRUE;
-	q_deactivate (qf);
-
-	/* if the field disappears, then destroy is the result */
-	g_object_unref (G_OBJECT (qf));
+	if (query_view_get_activated (qv) == FALSE) {
+		q_deactivate (qf);
+		query_field_declare_nullified (qf);
+	}
 }
 
 static gchar     * 
@@ -270,7 +266,7 @@ q_render_as_sql   (QueryField *qf, GSList * missing_values)
 {
 	gchar *str = NULL;
 
-	if (qf->activated) 
+	if (qf->priv->activated) 
 		str = g_strdup_printf ("%s.*", query_view_get_name (QF_PRIVATE_DATA(qf)->view));
 
 	return str;
@@ -287,7 +283,7 @@ q_render_as_string(QueryField *qf, GSList * missing_values)
 {
 	gchar *str = NULL;
 
-	if (qf->activated) 
+	if (qf->priv->activated) 
 		str = g_strdup_printf ("%s.*", query_view_get_name (QF_PRIVATE_DATA(qf)->view));
 
 	return str;
@@ -296,7 +292,7 @@ q_render_as_string(QueryField *qf, GSList * missing_values)
 static void  
 q_save_to_xml     (QueryField *qf, xmlNodePtr node)
 {
-	if (qf->activated) {
+	if (qf->priv->activated) {
 		gchar *str;
 
 		/* node object ref */
@@ -356,7 +352,7 @@ q_is_equal_to (QueryField *qf, QueryField *other)
 {
 	gboolean retval = FALSE;;
 
-	if (qf->activated && other->activated) {
+	if (qf->priv->activated && other->priv->activated) {
 		if (QF_PRIVATE_DATA(qf)->view == QF_PRIVATE_DATA(other)->view)
 			retval = TRUE;
 	}
@@ -369,14 +365,14 @@ q_get_monitored_objects (QueryField *qf)
 {
 	GSList *list = NULL;
 
-	if (qf->activated)
+	if (qf->priv->activated)
 		list = g_slist_prepend (NULL, QF_PRIVATE_DATA(qf)->view);
 
 	return list;
 }
 
 static void
-q_replace_comp (QueryField *qf, gint ref, GObject   *old, GObject   *new)
+q_replace_comp (QueryField *qf, gint ref, GObject *old, GObject *new)
 {
 	/* References can be to QueryViews can change. The obj attribute of the
 	 new and the old QueryViews can either:
@@ -386,13 +382,14 @@ q_replace_comp (QueryField *qf, gint ref, GObject   *old, GObject   *new)
 
 	if ((G_OBJECT (QF_PRIVATE_DATA(qf)->view) == old) && 
 	    (IS_QUERY_VIEW (old) && IS_QUERY_VIEW (new))) {
-		if (QUERY_VIEW (old)->obj == QUERY_VIEW (new)->obj) { /* case 1 */
+		if (query_view_get_view_obj (QUERY_VIEW (old)) == 
+		    query_view_get_view_obj (QUERY_VIEW (new))) {
 			/* we can keep it activated because the field does not change */
-			g_object_weak_unref (G_OBJECT (QF_PRIVATE_DATA(qf)->view),
-					     (GWeakNotify) q_view_obj_destroyed_wr, qf);
+			g_signal_handlers_disconnect_by_func (G_OBJECT (old),
+							      G_CALLBACK (view_status_changed_cb), qf);
 			QF_PRIVATE_DATA(qf)->view = QUERY_VIEW (new);
-			g_object_weak_ref (G_OBJECT (QF_PRIVATE_DATA(qf)->view),
-					   (GWeakNotify) q_view_obj_destroyed_wr, qf);
+			g_signal_connect (G_OBJECT (new), "status_changed",
+					  G_CALLBACK (view_status_changed_cb), qf);
 		}
 		else 
 			query_field_allfields_set_queryview (qf, QUERY_VIEW (new));	
@@ -421,7 +418,7 @@ query_field_allfields_dbtable_new (Query *q, gchar * name, QueryView *qv, DbTabl
 	g_return_val_if_fail (q && IS_QUERY (q), NULL);
 	g_return_val_if_fail (table && IS_DB_TABLE (table), NULL);
 	if (qv) {
-		g_return_val_if_fail (IS_QUERY_VIEW (qv) && (qv->query == q), NULL);
+		g_return_val_if_fail (IS_QUERY_VIEW (qv) && (query_view_get_query (qv) == q), NULL);
 		realqv = qv;
 	}
 	else { 
@@ -431,8 +428,8 @@ query_field_allfields_dbtable_new (Query *q, gchar * name, QueryView *qv, DbTabl
 
 		list = q->views;
 		while (list && !qv2) {
-			if (IS_DB_TABLE (QUERY_VIEW (list->data)->obj) &&
-			    (DB_TABLE (QUERY_VIEW (list->data)->obj) == table))
+			if (IS_DB_TABLE (query_view_get_view_obj (QUERY_VIEW (list->data))) &&
+			    (DB_TABLE (query_view_get_view_obj (QUERY_VIEW (list->data))) == table))
 				qv2 = QUERY_VIEW (list->data);
 			list = g_slist_next (list);
 		}
@@ -460,7 +457,7 @@ QueryField *qf;
 	g_return_val_if_fail (q && IS_QUERY (q), NULL);
 	g_return_val_if_fail (query && IS_QUERY (query), NULL);
 	if (qv) {
-		g_return_val_if_fail (IS_QUERY_VIEW (qv) && (qv->query == q), NULL);
+		g_return_val_if_fail (IS_QUERY_VIEW (qv) && (query_view_get_query (qv) == q), NULL);
 		realqv = qv;
 	}
 	else { 
@@ -470,8 +467,8 @@ QueryField *qf;
 
 		list = q->views;
 		while (list && !qv2) {
-			if (IS_QUERY (QUERY_VIEW (list->data)->obj) &&
-			    (QUERY (QUERY_VIEW (list->data)->obj) == query))
+			if (IS_QUERY (query_view_get_view_obj (QUERY_VIEW (list->data))) &&
+			    (QUERY (query_view_get_view_obj (QUERY_VIEW (list->data))) == query))
 				qv2 = QUERY_VIEW (list->data);
 			list = g_slist_next (list);
 		}
@@ -495,15 +492,15 @@ static void
 query_field_allfields_set_queryview (QueryField *qf, QueryView *qv)
 {
 	g_return_if_fail (qf && IS_QUERY_FIELD (qf));
-	g_return_if_fail (qf->field_type == QUERY_FIELD_ALLFIELDS);
+	g_return_if_fail (qf->priv->field_type == QUERY_FIELD_ALLFIELDS);
 	g_return_if_fail (qv && IS_QUERY_VIEW (qv));
 
-	if (qf->activated) 
+	if (qf->priv->activated) 
 		query_field_deactivate (qf);
 
 
 	QF_PRIVATE_DATA(qf)->view = qv;
-	qf->query = qv->query;
+	qf->priv->query = query_view_get_query (qv);
 
 	query_field_activate (qf);
 }
@@ -515,17 +512,17 @@ query_field_allfields_get_span (QueryField *qf)
 
 	g_assert (qf);
 	g_assert (IS_QUERY_FIELD (qf));
-	g_assert (qf->field_type == QUERY_FIELD_ALLFIELDS);
+	g_assert (qf->priv->field_type == QUERY_FIELD_ALLFIELDS);
 
-	if (qf->activated) {
-		if (IS_DB_TABLE (QF_PRIVATE_DATA(qf)->view->obj)) 
-			retval = g_slist_length (DB_TABLE (QF_PRIVATE_DATA(qf)->view->obj)->fields);
+	if (qf->priv->activated) {
+		if (IS_DB_TABLE (query_view_get_view_obj (QF_PRIVATE_DATA(qf)->view))) 
+			retval = g_slist_length (DB_TABLE (query_view_get_view_obj (QF_PRIVATE_DATA(qf)->view))->fields);
 		else {
 			GSList *list;
 			gint i=0;
-			list = QUERY (QF_PRIVATE_DATA(qf)->view->obj)->fields;
+			list = QUERY (query_view_get_view_obj (QF_PRIVATE_DATA(qf)->view))->fields;
 			while (list) {
-				if (QUERY_FIELD (list->data)->is_printed)
+				if (QUERY_FIELD (list->data)->priv->is_printed)
 					i++;
 				list = g_slist_next (list);
 			}
@@ -543,9 +540,9 @@ query_field_allfields_get_queryview (QueryField *qf)
 
 	g_assert (qf);
 	g_assert (IS_QUERY_FIELD (qf));
-	g_assert (qf->field_type == QUERY_FIELD_ALLFIELDS);
+	g_assert (qf->priv->field_type == QUERY_FIELD_ALLFIELDS);
 
-	if (qf->activated) 
+	if (qf->priv->activated) 
 		view = QF_PRIVATE_DATA(qf)->view;
 
 	return view;
