@@ -89,8 +89,7 @@ struct _QueryParamValue
 struct _Query
 {
 	GObject            object;
-
-	GSList            *weak_ref_objects;
+	gboolean           coherent; /* FALSE when temporary changes which should not be signaled */
 
 	/* parents and children */
 	Query             *parent;       /* to the parent query if it exists */
@@ -129,6 +128,7 @@ struct _QueryClass
 	void (*field_created)             (Query * q, QueryField * new_field);
 	void (*field_dropped)             (Query * q, QueryField * field);
 	void (*field_modified)            (Query * q, QueryField * field);
+	void (*field_print_modified)      (Query * q, QueryField * field);
 	void (*field_name_modified)       (Query * q, QueryField * field);
 	void (*field_alias_modified)      (Query * q, QueryField * field);
 
@@ -193,6 +193,7 @@ QueryView  *query_add_view_with_obj    (Query *q, GObject   *obj);
 void        query_del_view             (Query *q, QueryView *qv); /* qv gets free'd */
 
 void        query_add_join	       (Query *q, QueryView *v1, QueryField *f1, QueryView  *v2, QueryField *f2);
+void        query_add_join_and_fields  (Query *q, QueryView *v1, GObject *f1, QueryView  *v2, GObject *f2);
 void        query_del_join             (Query * q, QueryJoin *join);
 
 
@@ -229,7 +230,8 @@ gint        query_get_pos_by_field     (Query * q, QueryField *qf); /* -1 if err
 QueryField *query_get_field_by_name    (Query * q, gchar * name);
 QueryField *query_get_field_by_xmlid   (Query * q, gchar * xmlid);
 QueryField *query_get_field_by_id      (Query * q, guint id);
-QueryField *query_get_field_by_dbfield (Query * q, QueryView *qv, DbField *field);
+QueryField *query_get_field_by_dbfield    (Query * q, QueryView *qv, DbField *field);
+QueryField *query_get_field_by_queryfield (Query * q, QueryView *qv, QueryField *field); /* field is of another query */
 
 
 /* tells if a particular field appears in the print list of the Query:
@@ -266,17 +268,14 @@ void        query_dump_contents           (Query *q);
 #define QUERY_VIEWCLASS(klass)   G_TYPE_CHECK_CLASS_CAST (klass, query_view_get_type (), QueryViewClass)
 #define IS_QUERY_VIEW(obj)       G_TYPE_CHECK_INSTANCE_TYPE (obj, query_view_get_type ())
 
+typedef struct _QueryViewPrivate QueryViewPrivate;
+
 
 struct _QueryView
 {
-	GObject    object;
+	GObject           object;
 
-	Query     *query;
-	GObject   *obj;       /* Query or DbTable */
-	gint       occ;       /* occurrence in case of aliases, starts at 0 */
-	gchar     *alias;     /* new name given as alias, if any */
-	gchar     *alias_exp; /* new name given as alias, if any */
-	gchar     *ref;       /* may be used to find a reference to the Query */
+	QueryViewPrivate *priv;
 };
 
 
@@ -284,6 +283,10 @@ struct _QueryView
 struct _QueryViewClass
 {
 	GObjectClass   parent_class;
+
+	void         (*content_added)   (QueryView *qv, GObject new_item);
+	void         (*content_removed) (QueryView *qv, GObject old_item);
+	void         (*status_changed)  (QueryView *qv);
 };
 
 
@@ -294,10 +297,18 @@ struct _QueryViewClass
 guint        query_view_get_type           (void);
 GObject     *query_view_new                (Query *q);
 GObject     *query_view_new_copy           (QueryView *qv);
-void         query_view_free               (QueryView *qv);
 
 void         query_view_set_query          (QueryView *qv, Query *q);
 void         query_view_set_view_obj       (QueryView *qv, GObject *obj);
+void         query_view_set_occ            (QueryView *qv, gint occ);
+void         query_view_set_alias          (QueryView *qv, const gchar *alias);
+
+/* 
+ * Activation
+ */
+void         query_view_deactivate         (QueryView *qv);
+void         query_view_activate           (QueryView *qv);
+gboolean     query_view_get_activated      (QueryView *qv);
 	
 
 /*
@@ -312,13 +323,20 @@ void         query_view_load_from_xml      (QueryView *qv, xmlNodePtr node);
  * Getting information
  */
 QueryView   *query_view_find_from_xml_name (ConfManager * conf, Query * start_query, gchar *xmlname);
+
+GObject     *query_view_get_view_obj       (QueryView *qv);
+Query       *query_view_get_query          (QueryView *qv);
+gint         query_view_get_occ            (QueryView *qv);
+gboolean     query_view_get_is_activated   (QueryView *qv);
+
 gchar       *query_view_get_textual        (QueryView *qv);
 const gchar *query_view_get_name	   (QueryView *qv); 
 const gchar *query_view_get_real_name	   (QueryView *qv);
 gchar       *query_view_get_field_name     (QueryView *qv, GObject   *qvf);
-gchar 	    *query_view_get_alias_exp	   (QueryView *qv);
+const gchar *query_view_get_alias_expr	   (QueryView *qv);
+const gchar *query_view_get_alias	   (QueryView *qv);
 gboolean     query_view_is_alias	   (QueryView *qv);
-gboolean     query_view_contains_field     (QueryView *qv, GObject *field); /* field is DbFieldor or QueryField */
+gboolean     query_view_contains_field     (QueryView *qv, GObject *field); /* field is DbField or or QueryField */
 
 
 
@@ -343,95 +361,27 @@ gboolean     query_view_contains_field     (QueryView *qv, GObject *field); /* f
 #define QUERY_FIELD_CLASS(klass)  G_TYPE_CHECK_CLASS_CAST (klass, query_field_get_type (), QueryFieldClass)
 #define IS_QUERY_FIELD(obj)       G_TYPE_CHECK_INSTANCE_TYPE (obj, query_field_get_type ())
 
-typedef struct _QueryFieldIface QueryFieldIface;
+typedef struct _QueryFieldPrivate QueryFieldPrivate;
+typedef enum   _QueryFieldType QueryFieldType;
 
-typedef enum
-	{
-		QUERY_FIELD_FIELD,
-		QUERY_FIELD_ALLFIELDS,
-		QUERY_FIELD_AGGREGATE,
-		QUERY_FIELD_FUNCTION,
-		QUERY_FIELD_VALUE,
-		QUERY_FIELD_QUERY,
-		QUERY_FIELD_QUERY_FIELD,
-		QUERY_FIELD_LAST
-	}
-QueryFieldType;
-
-/*
- * Description of the common interface of all the
- * QueryFields:
- * - init():
- *   will initialise any private data, connect signals, etc
- *   for the QueryField type; the QueryField object has already 
- *   been initialised by the GTK object system
- * - destroy():
- *   will only free any memory, signal handlers, etc from the
- *   object, not free the object itself
- * - get_edit_widget():
- *   returns a widget to edit the contents of a QueryField
- * - get_sel_widget():
- *   returns a widget to enable the selection of the QueryField but also
- *   of any QueryField which is being used by this one.
- *   This function MUST connect a selection event to the "func" callback and
- *   MUST set the "qf" property to the widget emitting that signal (as well as
- *   optionnaly the "pqf" or "ref" ones).
- * - get_monitored_objects():
- *   returns a list (to be freed by the caller)
- *   with all the objects the QueryField receives signals from.
- * - render_as_string():
- *   will give a string describng the query field
- *   the missing_values argument can always be NULL
- * - get_xml_id():
- *   will return the ID part of the XML node where the QueryField
- *   saves itself, so that other objects can reference it in their XML element.
- * - replace_comp():
- *   replace a QueryField with another (it is possible to give ref=-1 or old=NULL
- *   but not the two at the same time)
- *   
- */
-struct _QueryFieldIface {
-	QueryFieldType field_type;
-	gchar        * name; /* the name of the 'type' attribute in the XML file */
-	gchar        * pretty_name; /* A name understandable by the user */
-	void        (* init)            (QueryField *qf);
-	void        (* destroy)         (QueryField *qf);
-	void        (* deactivate)      (QueryField *qf);
-	void        (* activate)        (QueryField *qf);
-	GtkWidget * (* get_edit_widget) (QueryField *qf);
-	GtkWidget * (* get_sel_widget)  (QueryField *qf, GCallback callback, gpointer data);
-	gchar     * (* render_as_sql)   (QueryField *qf, GSList * missing_values);
-	xmlNodePtr  (* render_as_xml)   (QueryField *qf, GSList * missing_values);
-	gchar     * (* render_as_string)(QueryField *qf, GSList * missing_values);
-	void        (* save_to_xml)     (QueryField *qf, xmlNodePtr node);
-	void        (* load_from_xml)   (QueryField *qf, xmlNodePtr node);
-	void        (* copy_other_field)(QueryField *qf, QueryField *other);
-	gboolean    (* is_equal_to)     (QueryField *qf, QueryField *other);
-	GSList    * (* get_monitored_objects) (QueryField *qf);
-	void        (* replace_comp)    (QueryField *qf, gint ref, GObject   *old, GObject   *new);
+enum _QueryFieldType{
+	QUERY_FIELD_FIELD,
+	QUERY_FIELD_ALLFIELDS,
+	QUERY_FIELD_AGGREGATE,
+	QUERY_FIELD_FUNCTION,
+	QUERY_FIELD_VALUE,
+	QUERY_FIELD_QUERY,
+	QUERY_FIELD_QUERY_FIELD,
+	QUERY_FIELD_LAST
 };
 
+
 /* struct for the object's data */
 struct _QueryField
 {
-	GObject   object;
+	GObject            object;
 
-	Query         *query;
-	gboolean       is_printed;   /* TRUE if it appears in the resulting columns */
-	gchar         *name;
-	gchar         *alias;
-        gboolean       is_hidden;    /* TRUE if it is printed but has been added by Mergeant itself */
-	QueryFieldType field_type;
-		
-	guint          id; /* Id unique only inside the Query */
-		
-	/* TRUE if the object has references to all the objects
-	   it wants to receive events from */
-	gboolean       activated;
-		
-	/* private data which can be used by the different query field types */
-	gpointer       private_data;
-	guint          ref_counter;
+	QueryFieldPrivate *priv;
 };
 
 /* struct for the object's class */
@@ -441,9 +391,11 @@ struct _QueryFieldClass
 
 	void (*field_modified)     (QueryField *qf);
 	void (*field_type_changed) (QueryField *qf);
+	void (*print_changed)      (QueryField *qf);
 	void (*name_changed)       (QueryField *qf);
 	void (*alias_changed)      (QueryField *qf);
 	void (*status_changed)     (QueryField *qf);
+	void (*nullified)          (QueryField *qf);
 
 	/* list of query field types, static to the class */
 	GSList *field_types; /* of type QueryFieldIface */
@@ -452,67 +404,72 @@ struct _QueryFieldClass
 /* 
  * Object's management
  */
-guint       query_field_get_type          (void);
-GObject    *query_field_new               (Query *q, gchar * name, QueryFieldType field_type);
-void        query_field_free              (QueryField *field);
-GObject    *query_field_new_objref        (Query *q, gchar * name, gchar *type); 
-GObject    *query_field_new_copy          (QueryField *field); /* new QF name is NULL */
+guint          query_field_get_type          (void);
+void           query_field_free              (QueryField *field);
+GObject       *query_field_new_objref        (Query *q, gchar * name, gchar *type); 
+GObject       *query_field_new_copy          (QueryField *field); /* new QF name is NULL */
 /* create a copy of the object and of all the other objects
    related to that object (recursively). It is NOT possible to pass NULL
    as list (this param returns the list of QueryField created).
    (*list) MUST be NULL or everything in the list will be lost */
-GObject    *query_field_new_copy_all      (QueryField *field, GSList **list);
+GObject       *query_field_new_copy_all      (QueryField *field, GSList **list);
 
 
 /*
  * Contents modifications
  */
-void        query_field_set_name          (QueryField * qf, const gchar * name);
-void        query_field_set_alias         (QueryField * qf, const gchar * alias);
-void        query_field_set_is_printed    (QueryField * qf, gboolean is_printed);
-void        query_field_set_is_hidden     (QueryField * qf, gboolean is_hidden);
-void        query_field_set_id            (QueryField * qf, guint id);
+void           query_field_set_name          (QueryField * qf, const gchar * name);
+void           query_field_set_alias         (QueryField * qf, const gchar * alias);
+void           query_field_set_is_printed    (QueryField * qf, gboolean is_printed);
+void           query_field_set_is_hidden     (QueryField * qf, gboolean is_hidden);
+void           query_field_set_id            (QueryField * qf, guint id);
 /* copies the contents of a QueryField into another alredy existing */
-void        query_field_copy_object       (QueryField *dest, QueryField *orig);
+void           query_field_copy_object       (QueryField *dest, QueryField *orig);
 
-
-/*
- * specific referencing 
- * FIXME: NOT USED Counter's management (except in func.c)
- */
-void        query_field_use_ref           (QueryField *field);
-void        query_field_free_ref          (QueryField *field);
 	
 /* 
  * Activation
  */
-void        query_field_deactivate        (QueryField * qf);
-void        query_field_activate          (QueryField * qf);
-void        query_field_get_activated     (QueryField * qf);
+void           query_field_deactivate        (QueryField * qf);
+void           query_field_activate          (QueryField * qf);
+gboolean       query_field_get_activated     (QueryField * qf);
 
 
 /*
  * Getting information / using the QF
  */
-gboolean    query_field_is_equal          (QueryField *field, QueryField *other);
-GtkWidget * query_field_get_edit_widget   (QueryField *qf);
-GtkWidget * query_field_get_select_widget (QueryField *qf, GtkSignalFunc func, gpointer data);
+QueryFieldType query_field_get_ftype         (QueryField *qf);
+const gchar   *query_field_get_ftype_name    (QueryField *qf);
 
-gchar     * query_field_render_as_sql     (QueryField *qf, GSList * missing_values);
-xmlNodePtr  query_field_render_as_xml     (QueryField *qf, GSList * missing_values);
-gchar     * query_field_render_as_string  (QueryField *qf, GSList * missing_values);
+guint          query_field_get_id            (QueryField *qf);
+Query         *query_field_get_query         (QueryField *qf);
+const gchar   *query_field_get_name          (QueryField *qf);
+const gchar   *query_field_get_alias         (QueryField *qf);
+gchar         *query_field_get_sql_alias     (QueryField *qf);
+gboolean       query_field_get_is_printed    (QueryField *qf);
+gboolean       query_field_get_is_hidden     (QueryField *qf);
 
-GSList    * query_field_get_monitored_objects (QueryField *qf); /* free the returned list */
 
-void        query_field_replace_ref_ptr   (QueryField *qf, GObject   *old, GObject   *new);
-void        query_field_replace_ref_int   (QueryField *qf, gint ref, QueryField *new);
+gboolean       query_field_is_equal          (QueryField *field, QueryField *other);
+GtkWidget     *query_field_get_edit_widget   (QueryField *qf);
+GtkWidget     *query_field_get_select_widget (QueryField *qf, QueryField *sel_qf,
+					      GtkSignalFunc func, gpointer data);
+
+gchar         *query_field_render_as_sql     (QueryField *qf, GSList * missing_values);
+xmlNodePtr     query_field_render_as_xml     (QueryField *qf, GSList * missing_values);
+gchar         *query_field_render_as_string  (QueryField *qf, GSList * missing_values);
+
+GSList        *query_field_get_monitored_objects (QueryField *qf); /* free the returned list */
+
+void           query_field_replace_ref_ptr   (QueryField *qf, GObject *old, GObject *new);
+void           query_field_replace_ref_int   (QueryField *qf, gint ref, QueryField *new);
 
 /*
  * XML manipulation
  */
-xmlNodePtr  query_field_save_to_xml       (QueryField *qf);
-void        query_field_load_from_xml     (QueryField *qf, xmlNodePtr node);
-gchar     * query_field_get_xml_id        (QueryField *qf);
+xmlNodePtr     query_field_save_to_xml       (QueryField *qf);
+void           query_field_load_from_xml     (QueryField *qf, xmlNodePtr node);
+gchar         *query_field_get_xml_id        (QueryField *qf);
 
 
 /*
@@ -533,34 +490,33 @@ gchar     * query_field_get_xml_id        (QueryField *qf);
 /* qv == NULL => create a new QueryView if necessary */
 QueryField      * query_field_field_dbfield_new       (Query *q, gchar * name, 
 						       QueryView *qv, DbField *field);
-QueryFieldIface * query_field_field_get_iface         ();
+QueryField      * query_field_field_queryfield_new    (Query *q, gchar * name, 
+						       QueryView *qv, QueryField *field);
 ServerDataType  * query_field_field_get_data_type     (QueryField *qf);
 /* qv can be NULL */
 QueryView       * query_field_field_get_queryview     (QueryField *qf);
+gboolean          query_field_field_is_db_field       (QueryField *qf);
 DbField         * query_field_field_get_db_field      (QueryField *qf);
-
+QueryField      * query_field_field_get_query_field   (QueryField *qf);
 
 /* Representation of all the fields of a table or a view, or a Query */
 QueryField      * query_field_allfields_dbtable_new   (Query *q, gchar * name, 
 						       QueryView *qv, DbTable *table);
 QueryField      * query_field_allfields_query_new     (Query *q, gchar * name, 
 						       QueryView *qv, Query *query);
-QueryFieldIface * query_field_allfields_get_iface     ();
 QueryView       * query_field_allfields_get_queryview (QueryField *qf);
 gint              query_field_allfields_get_span      (QueryField *qf);
 
 
 /* Representation of a value */
-QueryFieldIface * query_field_value_get_iface         ();
 ServerDataType  * query_field_value_get_data_type     (QueryField *qf);
 const GdaValue  * query_field_value_get_default_value (QueryField *qf);
-gboolean          query_field_value_input_required    (QueryField *qf);
+gboolean          query_field_value_is_parameter      (QueryField *qf);
+void              query_field_value_set_value         (QueryField *qf, const GdaValue *value);
 
 /* Representation of a function */
-QueryFieldIface * query_field_function_get_iface      ();
 
 /* Representation of a whole query */
-QueryFieldIface * query_field_query_get_iface         ();
 Query           * query_field_query_get_query         (QueryField *qf);
 
 /* Representation of another query's field */
@@ -588,6 +544,8 @@ Query           * query_field_query_get_query         (QueryField *qf);
 #define IS_QUERY_JOIN(obj)       G_TYPE_CHECK_INSTANCE_TYPE (obj, query_join_get_type ())
 #define QUERY_JOIN_PAIR_CAST(ptr) ((QueryJoinPair*)ptr)
 
+typedef struct _QueryJoinPrivate QueryJoinPrivate;
+
 /* Type of join between two tables (views) */
 enum _QueryJoinType {
 	QUERY_JOIN_INNER,
@@ -606,45 +564,13 @@ enum _QueryJoinCard {
 	LAST_QUERY_JOIN_CARD
 };
 
-/* pairs of (table's or query's) fields composing the join */
-typedef struct _QueryJoinPair QueryJoinPair;
-struct _QueryJoinPair {
-	QueryJoin     *qj;
-	GObject       *ant_field;
-	GObject       *suc_field;
-};
-#define QUERY_JOIN_PAIR(X) ((QueryJoinPair *) X)
 
 /* struct for the object's data */
 struct _QueryJoin
 {
-	GObject   object;
-
-	Query         *query;
-	QueryJoinType  join_type;
-		
-	/* tables or queries for the join (the object is either a DbTable or a Query) */
-	QueryView     *ant_view;
-	QueryView     *suc_view;
-
-	/* pairs of fields in the join (QueryJoinPair) */
-	GSList        *pairs;         /* list of field pairs being linked */
-	gchar         *condition;     /* holds the join condition */
-	gboolean       cond_modified; /* condition was modified by the user? */
-	/* FIXME?: join condition should be stored as an expression tree? */
-
-	/* List of weak referenced objects */
-	GSList        *weak_ref_objects;
-
-	/* cardinality of the join */
-	QueryJoinCard  card;
-
-	/* referential integrity */
-	gboolean       ref_integrity;
-	gboolean       cascade_update;
-	gboolean       cascade_delete;
-
-	gboolean       skip_view;    /* deal with duplicate suc_views (appear in loops) */
+	GObject           object;
+	
+	QueryJoinPrivate *priv;
 };
 
 
@@ -653,55 +579,66 @@ struct _QueryJoinClass
 {
 	GObjectClass   parent_class;
 
-	void (*type_changed) (QueryJoin *qj);
-	void (*card_changed) (QueryJoin *qj);
-	void (*pair_added)   (QueryJoin *qj, QueryJoinPair *pair);
-	void (*pair_removed) (QueryJoin *qj, QueryJoinPair *pair);
+	void (*nullified)      (QueryJoin *qj);
+	void (*type_changed)   (QueryJoin *qj);
+	void (*card_changed)   (QueryJoin *qj);
+	void (*cond_changed)   (QueryJoin *qj);
 };
 
+/* pairs of (table's or query's) fields composing the join's condition */
+typedef struct _QueryJoinPair QueryJoinPair;
+struct _QueryJoinPair {
+        GObject       *field1; /* objects are either DbField or QueryField */
+        GObject       *field2;
+};
+#define QUERY_JOIN_PAIR(X) ((QueryJoinPair *) X)
+
 /* 
  * Object's management
  */
-guint          query_join_get_type          (void);
-GObject       *query_join_new               (Query *q, QueryView *ant_view, QueryView*suc_view);
-QueryJoin     *query_join_copy              (QueryJoin *qj);
-void           query_join_free              (QueryJoin *qj);
+guint          query_join_get_type               (void);
+GObject       *query_join_new                    (Query *q, QueryView *ant_view, QueryView *suc_view);
+QueryJoin     *query_join_copy                   (QueryJoin *qj);
 
 
 /*
  * Setting contents
  */
-void           query_join_add_pair          (QueryJoin *qj, GObject   *field1, GObject   *field2);
-void           query_join_del_pair          (QueryJoin *qj, QueryJoinPair *pair);
-void           query_join_set_join_type     (QueryJoin *qj, QueryJoinType jt);
-void           query_join_set_card          (QueryJoin *qj, QueryJoinCard card);
-void	       query_join_swap_views	    (QueryJoin *qj);
-void	       query_join_set_condition     (QueryJoin *qj, gchar *cond);
+void           query_join_add_cond_fields        (QueryJoin *qj, QueryField *field1, QueryField *field2);
+void           query_join_add_cond               (QueryJoin *qj, QueryCond *cond);
+void           query_join_del_cond               (QueryJoin *qj, QueryCond *cond);
+void           query_join_set_join_type          (QueryJoin *qj, QueryJoinType jt);
+void           query_join_set_card               (QueryJoin *qj, QueryJoinCard card);
+void	       query_join_swap_views	         (QueryJoin *qj);
 
 /*
  * Getting information / using the QJ
 */
-QueryJoinType  query_join_get_join_type     (QueryJoin *qj);
-QueryJoinCard  query_join_get_card          (QueryJoin *qj);
-gchar	      *query_join_get_join_type_sql (QueryJoin *qj);
+QueryJoinType  query_join_get_join_type          (QueryJoin *qj);
+QueryJoinCard  query_join_get_card               (QueryJoin *qj);
+gchar	      *query_join_get_join_type_sql      (QueryJoin *qj);
 
-QueryView     *query_join_get_ant_view      (QueryJoin *qj);
-QueryView     *query_join_get_suc_view      (QueryJoin *qj);
-gchar	      *query_join_get_condition     (QueryJoin *qj);
+QueryView     *query_join_get_ant_view           (QueryJoin *qj);
+QueryView     *query_join_get_suc_view           (QueryJoin *qj);
+Query         *query_join_get_query              (QueryJoin *qj);
+QueryCond     *query_join_get_condition          (QueryJoin *qj);
 
-void	       query_join_set_skip_view     (QueryJoin *qj);
-void	       query_join_unset_skip_view   (QueryJoin *qj);
-gboolean       query_join_skip_view         (QueryJoin *qj);
+gboolean       query_join_cond_is_equi_only      (QueryJoin *qj);
+guint          query_join_cond_get_nb_equi_pairs (QueryJoin *qj);
+guint          query_join_cond_get_nb_conditions (QueryJoin *qj);
+GSList        *query_join_cond_get_equi_pairs    (QueryJoin *qj); /* list of QueryJoinPair, to free after usage */
 
+void	       query_join_set_skip_view          (QueryJoin *qj);
+void	       query_join_unset_skip_view        (QueryJoin *qj);
+gboolean       query_join_skip_view              (QueryJoin *qj);
 
-gchar         *query_join_render_as_sql     (QueryJoin *qj); 
-xmlNodePtr     query_join_render_as_xml     (QueryJoin *qj); 
-xmlNodePtr     query_join_save_to_xml       (QueryJoin *qj);
-GObject       *query_join_new_from_xml      (ConfManager *conf, xmlNodePtr node);
-xmlNodePtr     query_join_list_save_to_xml  (GSList *jlist);
+gchar         *query_join_render_as_sql          (QueryJoin *qj); 
+xmlNodePtr     query_join_render_as_xml          (QueryJoin *qj); 
+xmlNodePtr     query_join_save_to_xml            (QueryJoin *qj);
+GObject       *query_join_new_from_xml           (ConfManager *conf, xmlNodePtr node);
 
 #ifdef debug
-void           query_join_print             (QueryJoin *qj); /* DEBUG FER */
+void           query_join_print                  (QueryJoin *qj); /* DEBUG FER */
 #endif
 
 
@@ -723,17 +660,20 @@ void           query_join_print             (QueryJoin *qj); /* DEBUG FER */
 #define QUERY_COND_CLASS(klass)  G_TYPE_CHECK_CLASS_CAST (klass, query_cond_get_type (), QueryCondClass)
 #define IS_QUERY_COND(obj)       G_TYPE_CHECK_INSTANCE_TYPE (obj, query_cond_get_type ())
 
+typedef struct _QueryCondPrivate QueryCondPrivate;
+typedef enum   _QueryCondType    QueryCondType;
+typedef enum   _QueryCondOpType  QueryCondOpType;
 
 /* the different kinds of QueryCond objects */
-typedef enum {
+enum _QueryCondType {
 	QUERY_COND_COND,
 	QUERY_COND_AND,
 	QUERY_COND_OR,
 	QUERY_COND_NOT,
 	LAST_QUERY_COND_TYPE
-} QueryCondType;
+};
 
-typedef enum {
+enum _QueryCondOpType {
 	QUERY_COND_OP_EQUAL,
 	QUERY_COND_OP_DIFF,
 	QUERY_COND_OP_SUP,
@@ -745,25 +685,14 @@ typedef enum {
 	QUERY_COND_OP_IN,
 	QUERY_COND_OP_BETWEEN,
 	LAST_QUERY_COND_OP
-} QueryCondOpType;
-
+};
 
 /* struct for the object's data */
 struct _QueryCond
 {
-	GObject          object;
+	GObject           object;
 
-	Query           *query;
-	QueryCondType    cond_type;
-		
-	/* sub QueryCond objects if cond_type != QUERY_COND_COND */
-	GSList          *children;
-
-	/* if cond_type == QUERY_COND_COND */
-	QueryField      *left_op;
-	QueryField      *right_op;
-	QueryField      *right_op2;
-	QueryCondOpType  op_type;
+	QueryCondPrivate *priv;
 };
 
 /* struct for the object's class */
@@ -771,48 +700,58 @@ struct _QueryCondClass
 {
 	GObjectClass   parent_class;
 
-	void (* changed) (QueryCond *qc);
+	void        (*nullified) (QueryCond *qc);
+	void        (*changed)   (QueryCond *qc);
 };
 
 
 /*
  * Object's management
 */
-guint       query_cond_get_type         (void);
-GObject    *query_cond_new_as_node      (Query *q, QueryCondType type);
-GObject    *query_cond_new_as_leaf      (Query *q, QueryCondOpType type, 
-					 QueryField *left_op, QueryField *right_op, QueryField *right_op2);
-GObject    *query_cond_new_copy         (QueryCond *qc);
-void        query_cond_free             (QueryCond *qc);
+guint            query_cond_get_type         (void);
+GObject         *query_cond_new_as_node      (Query *q, QueryCondType type);
+GObject         *query_cond_new_as_leaf      (Query *q, QueryCondOpType type, 
+					      QueryField *left_op, QueryField *right_op, QueryField *right_op2);
+GObject         *query_cond_new_copy         (QueryCond *qc);
 
 /*
- * Contents setting
+ * Contents management
 */
-void        query_cond_add_to_node      (QueryCond *qc, QueryCond *child, gint pos); /* -1 => at the end */
-void        query_cond_del_from_node    (QueryCond *qc, QueryCond *child, gboolean destroy_child);
-void        query_cond_set_leaf_type    (QueryCond *qc, QueryCondOpType type);
+void             query_cond_add_to_node      (QueryCond *qc, QueryCond *child, gint pos); /* -1 => at the end */
+void             query_cond_del_from_node    (QueryCond *qc, QueryCond *child);
+void             query_cond_set_leaf_type    (QueryCond *qc, QueryCondOpType type);
 
 
 /*
  *  Getting information / using the QC
 */
-gboolean    query_cond_is_ancestor      (QueryCond *qc, QueryCond *ancestor);
-QueryCond  *query_cond_get_parent       (QueryCond *qc, QueryCond *top_cond);
-gchar     * query_cond_render_as_sql    (QueryCond *qc, GSList * missing_values);
-xmlNodePtr  query_cond_render_as_xml    (QueryCond *qc, GSList * missing_values);
+QueryCondType    query_cond_get_cond_type    (QueryCond *qc);
+QueryCondOpType  query_cond_get_op_type      (QueryCond *qc);
+Query           *query_cond_get_query        (QueryCond *qc);
+gboolean         query_cond_is_ancestor      (QueryCond *qc, QueryCond *ancestor);
+QueryCond       *query_cond_get_parent       (QueryCond *qc);
+GSList          *query_cond_get_children     (QueryCond *qc);
+GSList          *query_cond_get_query_fields (QueryCond *qc); /* free the list afterwards */
+QueryField      *query_cond_get_op_left      (QueryCond *qc);
+QueryField      *query_cond_get_op_right     (QueryCond *qc);
+QueryField      *query_cond_get_op_right2    (QueryCond *qc);
+
+
+gchar          * query_cond_render_as_sql    (QueryCond *qc, GSList * missing_values);
+xmlNodePtr       query_cond_render_as_xml    (QueryCond *qc, GSList * missing_values);
 
 
 /*
  * XML manipulations
 */
-xmlNodePtr  query_cond_save_to_xml      (QueryCond *qc);
-GObject    *query_cond_new_from_xml     (ConfManager *conf, Query *q, xmlNodePtr node);
+xmlNodePtr       query_cond_save_to_xml      (QueryCond *qc);
+GObject         *query_cond_new_from_xml     (ConfManager *conf, Query *q, xmlNodePtr node);
 
 
 
 #ifdef debug
 /* structure debug function */
-void        query_cond_dump_contents    (QueryCond *qc, guint offset);
+void             query_cond_dump_contents    (QueryCond *qc, guint offset);
 #endif
 
 G_END_DECLS
