@@ -587,8 +587,9 @@ server_access_do_query (ServerAccess * srv, const gchar * query, ServerAccessQue
 							      GTK_STOCK_YES, GTK_RESPONSE_YES,
 							      GTK_STOCK_NO, GTK_RESPONSE_NO, NULL);
 
-			str = g_strdup_printf (_("<span weight=\"bold\" size=\"larger\">Run this "
-						 "query?</span>\n\n%s"), query);
+			str = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
+					       _("Run this query?"),
+					       query);
 			label = gtk_label_new (NULL);
 			gtk_label_set_markup (GTK_LABEL (label), str);
 			g_free (str);
