@@ -21,7 +21,7 @@
 #include <config.h>
 #include <string.h>
 #include "server-access.h"
-#include "default-display.h"
+#include "data-handler.h"
 #include <libgda/libgda.h>
 
 /*
@@ -79,7 +79,7 @@ server_data_type_init (ServerDataType * dt)
 	dt->sqlname = NULL;
 	dt->numparams = 0;
 	dt->gda_type = 0;
-	dt->display_fns = NULL;
+	dt->data_handler = NULL;
 	dt->updated = FALSE;
 }
 
@@ -167,6 +167,13 @@ server_data_type_update_list (ServerAccess * srv)
 				server_data_type_set_descr (dt, str2);
 				g_free (str2);
 				srv->data_types = g_slist_append (srv->data_types, dt);
+#ifdef debug_signal
+				g_print (">> 'DATA_TYPE_ADDED' from server_data_type_update_list\n");
+#endif
+				g_signal_emit_by_name (G_OBJECT (srv), "data_type_added", dt);
+#ifdef debug_signal
+				g_print ("<< 'DATA_TYPE_ADDED' from server_data_type_update_list\n");
+#endif
 			}
 			g_free (str);
 
@@ -178,11 +185,9 @@ server_data_type_update_list (ServerAccess * srv)
 				dt->gda_type = gda_value_get_vtype (value);
 
 			/* functions to display the data type */
-			dt->display_fns = server_access_get_display_fns_from_gda (srv, dt);
+			dt->data_handler = server_access_get_handler_from_type (srv, dt);
 
-			g_signal_emit_by_name (G_OBJECT (srv), "progress",
-					       _("Updating the list of data types..."),
-					       now, total);
+			g_signal_emit_by_name (G_OBJECT (srv), "progress", "DATA_TYPES", now, total);
 			now++;
 		}
 		g_object_unref (G_OBJECT (rs));
@@ -197,13 +202,22 @@ server_data_type_update_list (ServerAccess * srv)
 			else {
 				GSList *hlist, *hold;
 				dt = SERVER_DATA_TYPE (dtl->data);
-				g_object_unref (G_OBJECT (dt));
 
 				hold = dtl;
 				hlist = g_slist_next (dtl);
 				srv->data_types = g_slist_remove_link (srv->data_types, dtl);
 				g_slist_free_1 (hold);
 				dtl = hlist;
+
+				g_object_unref (G_OBJECT (dt));
+#ifdef debug_signal
+				g_print (">> 'DATA_TYPE_REMOVED' from server_data_type_update_list\n");
+#endif
+				g_signal_emit_by_name (G_OBJECT (srv), "data_type_removed", dt);
+#ifdef debug_signal
+				g_print ("<< 'DATA_TYPE_REMOVED' from server_data_type_update_list\n");
+#endif
+
 			}
 		}
 
@@ -557,13 +571,11 @@ server_function_update_list (ServerAccess * srv)
 				}
 				srv->data_functions = g_slist_insert (srv->data_functions, df, i);
 #ifdef debug_signal
-				g_print (">> 'DATA_FUNCTION_ADDED' from "
-					 "server_function_update_list\n");
+				g_print (">> 'DATA_FUNCTION_ADDED' from server_function_update_list\n");
 #endif
 				g_signal_emit_by_name (G_OBJECT (srv), "data_function_added", df);
 #ifdef debug_signal
-				g_print ("<< 'DATA_FUNCTION_ADDED' from "
-					 "server_function_update_list\n");
+				g_print ("<< 'DATA_FUNCTION_ADDED' from server_function_update_list\n");
 #endif
 				df->updated = TRUE;
 			}
@@ -571,7 +583,7 @@ server_function_update_list (ServerAccess * srv)
 				g_slist_free (dtl);
 
 			g_signal_emit_by_name (G_OBJECT (srv), "progress",
-						 _("Updating functions list..."),
+						 "FUNCTIONS",
 						 now, total);
 			now ++;
 		}
@@ -609,12 +621,12 @@ server_function_update_list (ServerAccess * srv)
 
 	/* tell the world that we updated the functions */
 #ifdef debug_signal
-	g_print (">> 'DATA_FUNCTION_UPDATED' from "
+	g_print (">> 'DATA_FUNCTIONS_UPDATED' from "
 		 "server_function_update_list\n");
 #endif
-	g_signal_emit_by_name (G_OBJECT (srv), "data_function_updated");
+	g_signal_emit_by_name (G_OBJECT (srv), "data_functions_updated");
 #ifdef debug_signal
-	g_print ("<< 'DATA_FUNCTION_UPDATED' from "
+	g_print ("<< 'DATA_FUNCTIONS_UPDATED' from "
 		 "server_function_update_list\n");
 #endif
 	g_signal_emit_by_name (G_OBJECT (srv), "progress", NULL, 0, 0);
@@ -848,6 +860,8 @@ server_aggregate_finalize (GObject   * object)
 		g_free (df->descr);
 	if (df->sqlname)
 		g_free (df->sqlname);
+	if (df->objectid)
+		g_free (df->objectid);
 
 	/* parent class */
 	sa_parent_class->finalize (object);
@@ -920,6 +934,7 @@ server_aggregate_update_list (ServerAccess * srv)
 				str = gda_value_stringify (value);
 				da->objectid = str;
 
+
 				/* comments */
 				value = gda_data_model_get_value_at (rs, 3, now);
 				if (!gda_value_is_null (value)) {
@@ -934,10 +949,13 @@ server_aggregate_update_list (ServerAccess * srv)
 				while (list && !found) {
 					if (strcmp (da->sqlname, SERVER_AGGREGATE (list->data)->sqlname) > 0)
 						j++;
-					else
+					else {
 						found = TRUE;
+						break;
+					}
 					list = g_slist_next (list);
 				}
+
 				srv->data_aggregates = g_slist_insert (srv->data_aggregates, da, j);
 #ifdef debug_signal
 				g_print (">> 'DATA_AGGREGATE_ADDED' from "
@@ -954,9 +972,7 @@ server_aggregate_update_list (ServerAccess * srv)
 				if (da && (da->arg_type == dt))	/* mark as updated */
 					da->updated = TRUE;
 			}
-			g_signal_emit_by_name (G_OBJECT (srv), "progress",
-						 _("Updating aggregates list..."),
-						 now, total);
+			g_signal_emit_by_name (G_OBJECT (srv), "progress", "AGGREGATES", now, total);
 			now++;
 		}
 		g_object_unref (G_OBJECT (rs));
@@ -991,12 +1007,12 @@ server_aggregate_update_list (ServerAccess * srv)
 	}
 
 #ifdef debug_signal
-	g_print (">> 'DATA_AGGREGATE_UPDATED' from "
+	g_print (">> 'DATA_AGGREGATES_UPDATED' from "
 		 "server_aggregate_update_list\n");
 #endif
-	g_signal_emit_by_name (G_OBJECT (srv), "data_aggregate_updated");
+	g_signal_emit_by_name (G_OBJECT (srv), "data_aggregates_updated");
 #ifdef debug_signal
-	g_print ("<< 'DATA_AGGREGATE_UPDATED' from "
+	g_print ("<< 'DATA_AGGREGATES_UPDATED' from "
 		 "server_aggregate_update_list\n");
 #endif
 	g_signal_emit_by_name (G_OBJECT (srv), "progress", NULL, 0, 0);
